#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the MarlinAnalysis.py file."""

from __future__ import absolute_import
import unittest
import os
from mock import mock_open, patch, MagicMock as Mock
from ILCDIRAC.Workflow.Modules.MarlinAnalysis import MarlinAnalysis
from Tests.Utilities.GeneralUtils import assertInImproved, \
    assertEqualsImproved, assertDiracFailsWith, assertDiracSucceeds, \
    assertDiracSucceedsWith, assertDiracSucceedsWith_equals, assertMockCalls
from Tests.Utilities.FileUtils import FileUtil
from DIRAC import S_OK, S_ERROR

__RCSID__ = "$Id$"

# pylint: disable=missing-docstring, protected-access

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.MarlinAnalysis'


class MarlinAnalysisFixture(object):
  """Contains the commonly used setUp and tearDown methods of the Tests."""

  def setUp(self):
    """set up the objects."""
    self.marAna = MarlinAnalysis()
    self.marAna.OutputFile = ""
    self.marAna.ignoremissingInput = False
    self.log_mock = Mock()
    self.patches = [patch('%s.LOG' % MODULE_NAME, new=self.log_mock)]

    for patcher in self.patches:
      patcher.start()

  def tearDown(self):
    """Clean up test objects."""
    del self.marAna
    for patcher in self.patches:
      patcher.stop()

# TODO: add case for undefined steering file
# pylint: disable=too-many-public-methods


class MarlinAnalysisTestCase(MarlinAnalysisFixture, unittest.TestCase):
  """Base class for the ProductionJob test cases."""

  def test_applicationspecificinputs(self):
    self.marAna.InputFile = ['firstEntry.txt']
    self.marAna.workflow_commons['ParametricInputSandbox'] = \
        ['addthis.slcio', 'anotherFile_butignoreme', 'lastfile.slcio']
    assertDiracSucceeds(self.marAna.applicationSpecificInputs(), self)
    assertEqualsImproved((self.marAna.InputFile, self.marAna.InputData),
                          (['firstEntry.txt', 'addthis.slcio',
                              'anotherFile_butignoreme', 'lastfile.slcio'],
                            []), self)

  def test_applicationspecificinputs_nolist(self):
    self.marAna.InputFile = ['dontleavemealone.txt']
    self.marAna.workflow_commons['ParametricInputSandbox'] = 'myEntryies;another_one.slcio'
    assertDiracSucceeds(self.marAna.applicationSpecificInputs(), self)
    assertEqualsImproved((self.marAna.InputFile, self.marAna.InputData),
                          (['dontleavemealone.txt', 'myEntryies',
                              'another_one.slcio'], []), self)

  def test_applicationspecificinputs_emptylist(self):
    self.marAna.InputFile = ['leavemealone.txt']
    self.marAna.workflow_commons['ParametricInputSandbox'] = []
    assertDiracSucceeds(self.marAna.applicationSpecificInputs(), self)
    assertEqualsImproved((self.marAna.InputFile, self.marAna.InputData),
                          (['leavemealone.txt'], []), self)

  def test_applicationspecificinputs_emptystring(self):
    self.marAna.InputFile = ['leavemealone2.txt']
    self.marAna.workflow_commons['ParametricInputSandbox'] = ''
    assertDiracSucceeds(self.marAna.applicationSpecificInputs(), self)
    assertEqualsImproved((self.marAna.InputFile, self.marAna.InputData),
                          (['leavemealone2.txt'], []), self)

  def test_resolveinput_productionjob1(self):
    self.marAna.workflow_commons["IS_PROD"] = True
    outputfile1 = "/dir/a_REC_.a"
    outputfile2 = "/otherdir/b_DST_.b"
    inputfile = "/inputdir/input_SIM_.i"
    self.marAna.workflow_commons["ProductionOutputData"] = ";".join([outputfile1, outputfile2, inputfile])
    assertDiracSucceedsWith_equals(self.marAna.applicationSpecificInputs(),
                                    "Parameters resolved", self)
    assertEqualsImproved((self.marAna.outputREC, self.marAna.outputDST,
                            self.marAna.InputFile),
                          ("a_REC_.a", "b_DST_.b", ["input_SIM_.i"]), self)

  def test_resolveinput_productionjob2(self):
    self.marAna.workflow_commons['ParametricInputSandbox'] = []
    self.marAna.workflow_commons["IS_PROD"] = False
    self.marAna.workflow_commons["PRODUCTION_ID"] = "123"
    self.marAna.workflow_commons["JOB_ID"] = 456
    assertDiracSucceedsWith_equals(self.marAna.applicationSpecificInputs(),
                                    "Parameters resolved", self)
    self.assertEqual(self.marAna.InputFile, [])
    self.assertEqual(self.marAna.InputData, [])

  def test_resolveinput_productionjob3(self):
    self.marAna.workflow_commons["IS_PROD"] = True
    self.marAna.OutputFile = "c.c"
    self.marAna.InputFile = []
    inputlist = ["a.slcio", "b.slcio", "c.exe"]
    self.marAna.InputData = inputlist
    assertDiracSucceedsWith_equals(self.marAna.applicationSpecificInputs(),
                                    "Parameters resolved", self)
    self.assertEqual([inputlist[0], inputlist[1]], self.marAna.InputFile)

  def test_resolveinput_productionjob4(self):
    self.marAna.workflow_commons["IS_PROD"] = True
    self.marAna.OutputFile = ''
    prodId = "123"
    self.marAna.workflow_commons["PRODUCTION_ID"] = prodId
    jobId = "456"
    self.marAna.workflow_commons["JOB_ID"] = jobId
    self.marAna.workflow_commons.pop('ProductionOutputData', None)  # default value needed, else maybe KeyError
    self.marAna.InputFile = 'in3.slcio'
    self.marAna.outputREC = 'out1.stdhep'
    self.marAna.outputDST = 'test2.stdhep'
    assertDiracSucceedsWith_equals(self.marAna.applicationSpecificInputs(),
                                    "Parameters resolved", self)
    files = [self.marAna.outputREC, self.marAna.outputDST,
             self.marAna.InputFile[0]]
    for filename in files:
      # TODO: check for file extension, differentiate 'test'/'test2' in filename...
      self.assertInMultiple([prodId, jobId], filename)
    self.assertInMultiple(['in3', '.slcio'], self.marAna.InputFile[0])
    self.assertInMultiple(['out1', '.stdhep'], self.marAna.outputREC)
    self.assertInMultiple(['test2', '.stdhep'], self.marAna.outputDST)

  def test_runit_noplatform(self):
    self.marAna.platform = None
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'no ilc platform selected', self)

  def test_runit_noapplog(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = None
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'no log', self)

  def test_runit_workflowbad(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.workflowStatus = {'OK': False}
    result = self.marAna.runIt()
    assertDiracSucceedsWith(result, 'should not proceed', self)

  def test_runit_stepbad(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = {'OK': False}
    result = self.marAna.runIt()
    assertDiracSucceedsWith(result, 'should not proceed', self)

  @patch("%s.MarlinAnalysis._getDetectorXML" % MODULE_NAME,
         new=Mock(return_value=S_ERROR("zis iz not camelot")))
  def test_runit_detXMLFails(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = {'OK': True}
    self.marAna.detectorModel = "notCamelot.xml"
    result = self.marAna.runIt()
    assertDiracFailsWith(result, "zis iz not camelot", self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_ERROR("failed to get env script")))
  def test_runit_getEnvScriptFails(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'env script', self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK('Testpath123')))
  @patch("%s.MarlinAnalysis._getInputFiles" % MODULE_NAME, new=Mock(return_value=S_ERROR('missing slcio file')))
  def test_runit_getInputFilesFails(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    # Only checks that the patch annotation is passed as the result
    assertDiracFailsWith(result, 'missing slcio file', self)

  def test_runit_no_steeringfile(self):
    exists_dict = {'PandoraSettings.xml': False, 'testGear.input': False,
                    '/steeringdir/testGear.input': True,
                    '/steeringdir/PandoraSettings.xml': True, '': True,
                    'lib': True,
                  }

    def replace_exists(path):
      return exists_dict[path]
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    self.marAna.detectorModel = None
    self.marAna.inputGEAR = '/my/secret/path/testGear.input'
    self.marAna.SteeringFile = ''
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('envscript_path.test'))), \
         patch('%s.MarlinAnalysis._getInputFiles' % MODULE_NAME, new=Mock(return_value=S_OK(['testslcioList']))), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_OK('/steeringdir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)), \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('shutil_test_fail'))) as copy_mock, \
         patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/test/curdir')):
      result = self.marAna.runIt()
      assertDiracFailsWith(result, 'could not find steering file', self)
      self.log_mock.warn.assert_called_once_with('Could not copy PandoraSettings.xml, exception: shutil_test_fail')
      self.log_mock.error.assert_called_once_with("Steering file not defined, shouldn't happen!")
      copy_mock.assert_called_once_with('/steeringdir/PandoraSettings.xml',
                                         '/test/curdir/PandoraSettings.xml')

  def test_runit_no_steeringdir(self):
    exists_dict = {'PandoraSettings.xml': True, 'testGear.input': False,
                    '': True}

    def replace_exists(path):
      return exists_dict[path]
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    self.marAna.detectorModel = None
    self.marAna.inputGEAR = '/my/secret/path/testGear.input'
    self.marAna.SteeringFile = ''
    with patch('%s.getEnvironmentScript' % MODULE_NAME, new=Mock(return_value=S_OK('envscript_path.test'))), \
         patch('%s.MarlinAnalysis._getInputFiles' % MODULE_NAME, new=Mock(return_value=S_OK(['testslcioList']))), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_ERROR('mytesterr'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)), \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('shutil_test_fail'))) as copy_mock, \
         patch('%s.os.getcwd' % MODULE_NAME, new=Mock(return_value='/test/curdir')):
      result = self.marAna.runIt()
      assertDiracFailsWith(result, 'could not find steering file', self)
      self.log_mock.warn.assert_called_once_with('Could not find the steering file directory', 'mytesterr')
      self.log_mock.error.assert_called_once_with("Steering file not defined, shouldn't happen!")
      self.assertFalse(copy_mock.called)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_ERROR('')))
  def test_getenvscript_getsoftwarefolderfails(self):
    self.assertFalse(self.marAna.getEnvScript(None, None, None)['OK'])

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK('')))
  @patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.getNewLDLibs" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.os.path.exists" % MODULE_NAME, new=Mock(return_value=False))
  def test_getenvscript_pathexists(self):
    result = self.marAna.getEnvScript(None, None, None)
    assertDiracFailsWith(result, 'marlin_dll folder not found', self)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK('aFolder')))
  @patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.getNewLDLibs" % MODULE_NAME, new=Mock(return_value='bFolder'))
  @patch("%s.os.path.exists" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.os.listdir" % MODULE_NAME, new=Mock(return_value=['testdir']))
  def test_getenvscript(self):
    file_contents = []
    text_file_data = '\n'.join(file_contents)
    result = {}
    with patch('%s.open' % MODULE_NAME, mock_open(read_data=text_file_data), create=True) as file_mocker:
      result = self.marAna.getEnvScript(None, None, None)
    assertDiracSucceedsWith(result, '/MarlinEnv.sh', self)
    check_in_script = [
        'declare -x PATH=aFolder/Executable:$PATH\n', 'declare -x ROOTSYS=aFolder/ROOT\n',
        'declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:aFolder/LDLibs:bFolder\n',
        "declare -x MARLIN_DLL=aFolder/MARLIN_DLL/testdir:\n",
        "declare -x PANDORASETTINGS=aFolder/Settings/PandoraSettings.xml"]
    file_mocker.assert_called_with('MarlinEnv.sh', 'w')
    mocker_handle = file_mocker()
    for expected in check_in_script:
      mocker_handle.write.assert_any_call(expected)
    self.assertEqual(len(check_in_script),
                     mocker_handle.__enter__.return_value.write.call_count - 4)

  def test_getinputfiles_ignore(self):
    self.marAna.ignoremissingInput = True
    assertDiracSucceeds(self.marAna._getInputFiles(), self)

  def test_getinputfiles_resolvepathsfails(self):
    with patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_ERROR())):
      result = self.marAna._getInputFiles()
      assertDiracFailsWith(result, 'missing slcio', self)

  def test_getinputfiles_complete(self):
    with patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK(['1.slcio', '2.slcio']))):
      res = self.marAna._getInputFiles()
      assertDiracSucceedsWith_equals(res, '1.slcio 2.slcio', self)

  def assertInMultiple(self, listOfStrings, bigString):
    """Checks if every string in listOfStrings is contained in bigString."""
    for string in listOfStrings:
      assertInImproved(string, bigString, self)


class MarlinAnalysisPatchTestCase(MarlinAnalysisFixture, unittest.TestCase):
  """Contains test that have a certain set of patches of MarlinAnalysis methods in common."""

  def setUp(self):
    super(MarlinAnalysisPatchTestCase, self).setUp()
    thesePatches = [patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK('Testpath123'))),
                    patch("%s.MarlinAnalysis._getInputFiles" % MODULE_NAME,
                          new=Mock(return_value=S_OK("testinputfiles"))),
                    patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK('testdir'))),
                    patch("%s.os.path.exists" % MODULE_NAME,
                          new=Mock(side_effect=[False, True, False, True, False, True, False, False]))]
    self.patches.extend(thesePatches)
    for patcher in thesePatches:
      patcher.start()

  @patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.prepareMARLIN_DLL" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.runMarlin" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.shutil.copy" % MODULE_NAME)
  def test_runit_handlepandorasettings_no_log(self, mock_copy):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'not produce the expected log', self)
    mock_copy.assert_called_with('testdir/PandoraSettings.xml',
                                 '%s/PandoraSettings.xml' % os.getcwd())

  @patch("%s.shutil.copy" % MODULE_NAME, new=Mock())
  @patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_ERROR('prepxml')))
  def test_runit_xmlgenerationfails(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'prepxml', self)

  @patch("%s.shutil.copy" % MODULE_NAME, new=Mock())
  @patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.prepareMARLIN_DLL" % MODULE_NAME, new=Mock(return_value=S_ERROR('mardll')))
  def test_runit_marlindllfails(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'wrong with software installation', self)

  @patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.prepareMARLIN_DLL" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.runMarlin" % MODULE_NAME, new=Mock(return_value=S_ERROR('marlin failed')))
  @patch("%s.shutil.copy" % MODULE_NAME)
  def test_runit_runmarlinfails(self, mock_copy):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    result = self.marAna.runIt()
    assertDiracFailsWith(result, 'failed to run', self)
    mock_copy.assert_called_with('testdir/PandoraSettings.xml',
                                 '%s/PandoraSettings.xml' % os.getcwd())

  @patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.prepareMARLIN_DLL" % MODULE_NAME, new=Mock(return_value=S_OK('testdir')))
  @patch("%s.MarlinAnalysis.runMarlin" % MODULE_NAME, new=Mock(return_value=S_OK([""])))
  @patch("%s.shutil.copy" % MODULE_NAME)
  def test_runit_complete(self, mock_copy):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    with patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[False, True, False, True, True, True, True, True])):
      result = self.marAna.runIt()
      assertDiracSucceeds(result, self)
      mock_copy.assert_called_with('testdir/PandoraSettings.xml',
                                   '%s/PandoraSettings.xml' % os.getcwd())

  def test_runit_complete_dd(self):
    self.marAna.platform = "Testplatform123"
    self.marAna.applicationLog = "testlog123"
    self.marAna.stepStatus = S_OK()
    self.marAna.workflowStatus = S_OK()
    self.marAna.detectorModel = "someDetector.xml"
    with patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK('Testpath123'))), \
         patch("%s.MarlinAnalysis._getInputFiles" % MODULE_NAME, new=Mock(return_value=S_OK("testinputfiles"))), \
         patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK('testdir'))), \
         patch("%s.os.path.exists" % MODULE_NAME, new=Mock(side_effect=[False, True, False, True, True, True, True, True])), \
         patch("%s.prepareXMLFile" % MODULE_NAME, new=Mock(return_value=S_OK('testdir'))), \
         patch("%s.MarlinAnalysis.prepareMARLIN_DLL" % MODULE_NAME, new=Mock(return_value=S_OK('testdir'))), \
         patch("%s.MarlinAnalysis.runMarlin" % MODULE_NAME, new=Mock(return_value=S_OK([""]))), \
         patch("%s.shutil.copy" % MODULE_NAME) as mock_copy, \
         patch("%s.MarlinAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("someDetector.xml"))):
      result = self.marAna.runIt()
      assertDiracSucceeds(result, self)
      mock_copy.assert_called_with('testdir/PandoraSettings.xml',
                                   '%s/PandoraSettings.xml' % os.getcwd())

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_ERROR('')))
  def test_getenvscript_getsoftwarefolderfails(self):
    self.assertFalse(self.marAna.getEnvScript(None, None, None)['OK'])

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK('')))
  @patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.getNewLDLibs" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.os.path.exists" % MODULE_NAME, new=Mock(return_value=False))
  def test_getenvscript_pathexists(self):
    result = self.marAna.getEnvScript(None, None, None)
    assertDiracFailsWith(result, 'marlin_dll folder not found', self)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK('aFolder')))
  @patch("%s.removeLibc" % MODULE_NAME, new=Mock(return_value=None))
  @patch("%s.getNewLDLibs" % MODULE_NAME, new=Mock(return_value='bFolder'))
  @patch("%s.os.path.exists" % MODULE_NAME, new=Mock(return_value=True))
  @patch("%s.os.listdir" % MODULE_NAME, new=Mock(return_value=['testdir']))
  def test_getenvscript(self):
    file_contents = []
    text_file_data = '\n'.join(file_contents)
    result = {}
    with patch('%s.open' % MODULE_NAME, mock_open(read_data=text_file_data), create=True) as file_mocker:
      result = self.marAna.getEnvScript(None, None, None)
    assertDiracSucceedsWith(result, '/MarlinEnv.sh', self)
    check_in_script = [
        'declare -x PATH=aFolder/Executable:$PATH\n',
        'declare -x ROOTSYS=aFolder/ROOT\n',
        'declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:aFolder/LDLibs:bFolder\n',
        "declare -x MARLIN_DLL=aFolder/MARLIN_DLL/testdir:\n",
        "declare -x PANDORASETTINGS=aFolder/Settings/PandoraSettings.xml"]
    file_mocker.assert_called_with('MarlinEnv.sh', 'w')
    mocker_handle = file_mocker()
    for expected in check_in_script:
      mocker_handle.write.assert_any_call(expected)
    self.assertEqual(len(check_in_script),
                     mocker_handle.__enter__.return_value.write.call_count - 4)

  def assertInMultiple(self, listOfStrings, bigString):
    """Checks if every string in listOfStrings is contained in bigString."""
    for string in listOfStrings:
      assertInImproved(string, bigString, self)


class MarlinAnalysisPrepareDLLCase(MarlinAnalysisFixture, unittest.TestCase):
  """Tests for the prepareMARLIN_DLL method."""

  def test_preparemarlindll(self):
    exists_dict = {'./lib/marlin_dll': True}

    def replace_exists(path):
      return exists_dict[path]
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'MARlin_DLL/path: ', 'other_return_value_from_shell']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=[True])) as remove_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['mytestlibrary.so', 'secondLibrary.veryUseful.so'])) as glob_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      remove_mock.assert_called_once_with('temp.sh')
      exists_mock.assert_called_once_with('./lib/marlin_dll')
      glob_mock.assert_called_once_with('./lib/marlin_dll/*.so')
      assertDiracSucceedsWith(result, 'MARlin_DLL/path:mytestlibrary.so:secondLibrary.veryUseful.so',
                               self)

  def test_preparemarlindll_shellcall_fails(self):
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))) as shell_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      assertDiracFailsWith(result, 'failed getting the marlin_dll', self)

  def test_preparemarlindll_empty_marlindll(self):
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, '', 'other_value']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('something?'))):
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      assertDiracFailsWith(result, 'empty marlin_dll env var', self)

  def test_preparemarlindll_procstoinclude(self):
    self.marAna.ProcessorListToUse = ['secondLibrary.veryUseful.so']
    exists_dict = {'./lib/marlin_dll': True}

    def replace_exists(path):
      return exists_dict[path]
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'MARlin_DLL/path', 'other_return_value_from_shell']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=[True])) as remove_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['mytestlibrary.so', 'secondLibrary.veryUseful.so', 'MARlin_DLL/path'])) as glob_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      remove_mock.assert_called_once_with('temp.sh')
      exists_mock.assert_called_once_with('./lib/marlin_dll')
      glob_mock.assert_called_once_with('./lib/marlin_dll/*.so')
      assertDiracSucceedsWith(result, 'secondLibrary.veryUseful.so', self)

  def test_preparemarlindll_procstoexclude(self):
    self.marAna.ProcessorListToExclude = ['secondLibrary.veryUseful.so']
    exists_dict = {'./lib/marlin_dll': True}

    def replace_exists(path):
      return exists_dict[path]
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'MARlin_DLL/path', 'other_return_value_from_shell']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=[True])) as remove_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['mytestlibrary.so', 'secondLibrary.veryUseful.so'])) as glob_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      remove_mock.assert_called_once_with('temp.sh')
      exists_mock.assert_called_once_with('./lib/marlin_dll')
      glob_mock.assert_called_once_with('./lib/marlin_dll/*.so')
      assertDiracSucceedsWith(result, 'MARlin_DLL/path:mytestlibrary.so', self)

  def test_preparemarlindll_nolibs(self):
    exists_dict = {'./lib/marlin_dll': False}

    def replace_exists(path):
      return exists_dict[path]
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'MARlin_DLL/path: ', 'other_return_value_from_shell']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=[True])) as remove_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['mytestlibrary.so', 'secondLibrary.veryUseful.so'])) as glob_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      remove_mock.assert_called_once_with('temp.sh')
      exists_mock.assert_called_once_with('./lib/marlin_dll')
      self.assertFalse(glob_mock.called)
      assertDiracSucceedsWith(result, 'MARlin_DLL/path', self)

  def test_preparemarlindll_swaplibpositions(self):
    self.marAna.ProcessorListToExclude = ['mytestlibrary.so']
    exists_dict = {'./lib/marlin_dll': True}

    def replace_exists(path):
      return exists_dict[path]
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK([0, 'MARlin_DLL/path', 'other_return_value_from_shell']))) as shell_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(side_effect=[True])) as remove_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['testlibLCFIPlus.so', 'testlibLCFIVertex.1.so'])) as glob_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.marAna.prepareMARLIN_DLL('some_path')
      open_mock.assert_called_once_with('temp.sh', 'w')
      assertMockCalls(handles[0].__enter__().write,
                       ['#!/bin/bash\n', 'source some_path > /dev/null\necho $MARLIN_DLL'], self)
      assertEqualsImproved(len(handles), 1, self)
      chmod_mock.assert_called_once_with('temp.sh', 0o755)
      shell_mock.assert_called_once_with(0, './temp.sh')
      remove_mock.assert_called_once_with('temp.sh')
      exists_mock.assert_called_once_with('./lib/marlin_dll')
      glob_mock.assert_called_once_with('./lib/marlin_dll/*.so')
      assertDiracSucceedsWith(result, 'MARlin_DLL/path:testlibLCFIVertex.1.so:testlibLCFIPlus.so', self)


class MarlinAnalysisRunTestCase(MarlinAnalysisFixture, unittest.TestCase):
  """Tests for the runMarlin method."""

  def test_runmarlin(self):
    self.marAna.STEP_NUMBER = '935'
    self.marAna.applicationVersion = 'VT'
    self.marAna.debug = True
    self.marAna.applicationLog = 'testLog.mymarlin'
    self.marAna.extraCLIarguments = 'testMyArgs'
    exists_dict = {'./lib/lddlib': True, 'Marlin_VT_Run_935.sh': True,
                    './lib/marlin_dll': True, 'myXML.test': True, 'testLog.mymarlin': False,
                    './lib/': False,
                  }

    def replace_exists(path):
      return exists_dict[path]
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.os.remove' % MODULE_NAME) as remove_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK('shell succeeded'))) as shell_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.MarlinAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock:
      result = self.marAna.runMarlin('myXML.test', 'TestEnvscript.path', 'marlin_dll.test.so')
      assertDiracSucceedsWith_equals(result, 'shell succeeded', self)
      open_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source TestEnvscript.path\n', 'declare -x MARLIN_DLL=marlin_dll.test.so\n',
          'declare -x LD_LIBRARY_PATH=./lib/lddlib:$LD_LIBRARY_PATH\n',
          'declare -x PATH=$ROOTSYS/bin:$PATH\n', 'declare -x MARLIN_DEBUG=1\n',
          '\nif [ -e "${PANDORASETTINGS}" ]\nthen\n   cp $PANDORASETTINGS .\nfi\n',
          'echo =============================\n', 'echo LD_LIBRARY_PATH is\n',
          'echo $LD_LIBRARY_PATH | tr ":" "\n"\n',
          'echo =============================\n', 'echo PATH is\n',
          'echo $PATH | tr ":" "\n"\n', 'echo =============================\n',
          'echo MARLIN_DLL is\n', 'echo $MARLIN_DLL | tr ":" "\n"\n',
          'echo =============================\n', 'echo ldd of executable is\n',
          'ldd `which Marlin` \n', 'echo =============================\n',
          'ldd ./lib/marlin_dll/*.so \n', 'ldd ./lib/lddlib/*.so \n',
          'echo =============================\n', 'env | sort >> localEnv.log\n',
          'Marlin -c myXML.test testMyArgs\n', 'Marlin myXML.test testMyArgs\n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      remove_mock.assert_called_once_with('Marlin_VT_Run_935.sh')
      assertMockCalls(exists_mock, ['Marlin_VT_Run_935.sh', './lib/lddlib', './lib/marlin_dll',
                                      './lib/lddlib', 'myXML.test', 'testLog.mymarlin', './lib/'], self)
      shell_mock.assert_called_once_with(0, 'sh -c "./Marlin_VT_Run_935.sh"',
                                          callbackFunction=self.marAna.redirectLogOutput,
                                          bufferLimit=20971520)
      chmod_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 0o755)
      appstat_mock.assert_called_once_with('Marlin VT step 935')

  def test_runmarlin_nosteeringfile(self):
    self.marAna.STEP_NUMBER = '932'
    self.marAna.applicationVersion = 'V1T'
    self.marAna.debug = False
    self.marAna.extraCLIarguments = 'testMyArgs'
    exists_dict = {'./lib/lddlib': False, 'Marlin_V1T_Run_932.sh': False,
                    'inputxmlMy.test': False,
                    './lib/': False,
                  }

    def replace_exists(path):
      return exists_dict[path]
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)), \
         patch('%s.os.remove' % MODULE_NAME) as remove_mock, \
         patch('%s.shellCall' % MODULE_NAME) as shell_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.MarlinAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock:
      result = self.marAna.runMarlin('inputxmlMy.test', 'envscript.path',
                                      'marlin_dll')
      assertDiracFailsWith(result, 'steeringfile is missing', self)
      open_mock.assert_called_once_with('Marlin_V1T_Run_932.sh', 'w')
      open_mock().close.assert_called_once_with()
      self.assertFalse(remove_mock.called)
      self.assertFalse(shell_mock.called)
      self.assertFalse(appstat_mock.called)
      self.assertFalse(chmod_mock.called)

  def test_runmarlin_shell_fails(self):
    self.marAna.STEP_NUMBER = '935'
    self.marAna.applicationVersion = 'VT'
    self.marAna.debug = False
    self.marAna.applicationLog = 'testLog.mymarlin'
    self.marAna.extraCLIarguments = 'testMyArgs'
    exists_dict = {'./lib/lddlib': False, 'Marlin_VT_Run_935.sh': True,
                    'myXML.test': True, 'testLog.mymarlin': False,
                    './lib/': True,
                  }

    def replace_exists(path):
      return exists_dict[path]
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.os.remove' % MODULE_NAME) as remove_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err_with_script'))) as shell_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.MarlinAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock:
      result = self.marAna.runMarlin('myXML.test', 'TestEnvscript.path', 'marlin_dll.test.so')
      assertDiracFailsWith(result, 'some_test_err_with_script', self)
      open_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source TestEnvscript.path\n', 'declare -x MARLIN_DLL=marlin_dll.test.so\n',
          'declare -x LD_LIBRARY_PATH=./lib:$LD_LIBRARY_PATH\n',
          'declare -x PATH=$ROOTSYS/bin:$PATH\n', 'declare -x MARLIN_DEBUG=1\n',
          '\nif [ -e "${PANDORASETTINGS}" ]\nthen\n   cp $PANDORASETTINGS .\nfi\n',
          'echo =============================\n', 'echo LD_LIBRARY_PATH is\n',
          'echo $LD_LIBRARY_PATH | tr ":" "\n"\n', 'echo =============================\n',
          'echo PATH is\n', 'echo $PATH | tr ":" "\n"\n', 'echo =============================\n',
          'echo MARLIN_DLL is\n', 'echo $MARLIN_DLL | tr ":" "\n"\n', 'echo =============================\n',
          'env | sort >> localEnv.log\n', 'Marlin -c myXML.test testMyArgs\n', 'Marlin myXML.test testMyArgs\n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      remove_mock.assert_called_once_with('Marlin_VT_Run_935.sh')
      assertMockCalls(exists_mock, ['Marlin_VT_Run_935.sh', './lib/lddlib', 'myXML.test',
                                      'testLog.mymarlin', './lib/'], self)
      shell_mock.assert_called_once_with(0, 'sh -c "./Marlin_VT_Run_935.sh"',
                                          callbackFunction=self.marAna.redirectLogOutput,
                                          bufferLimit=20971520)
      chmod_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 0o755)
      appstat_mock.assert_called_once_with('Marlin VT step 935')

  def test_runmarlin_nolibs(self):
    self.marAna.STEP_NUMBER = '935'
    self.marAna.applicationVersion = 'VT'
    self.marAna.debug = True
    self.marAna.applicationLog = 'testLog.mymarlin'
    self.marAna.extraCLIarguments = 'testMyArgs'
    exists_dict = {'./lib/lddlib': False, 'Marlin_VT_Run_935.sh': True,
                    './lib/marlin_dll': False, 'myXML.test': True,
                    'testLog.mymarlin': False,
                    './lib/': False,
                  }

    def replace_exists(path):
      return exists_dict[path]
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.os.remove' % MODULE_NAME) as remove_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK('shell succeeded'))) as shell_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.MarlinAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock:
      result = self.marAna.runMarlin('myXML.test', 'TestEnvscript.path', 'marlin_dll.test.so')
      assertDiracSucceedsWith_equals(result, 'shell succeeded', self)
      open_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, [
          '#!/bin/bash \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'source TestEnvscript.path\n', 'declare -x MARLIN_DLL=marlin_dll.test.so\n',
          'declare -x PATH=$ROOTSYS/bin:$PATH\n', 'declare -x MARLIN_DEBUG=1\n',
          '\nif [ -e "${PANDORASETTINGS}" ]\nthen\n   cp $PANDORASETTINGS .\nfi\n',
          'echo =============================\n', 'echo LD_LIBRARY_PATH is\n',
          'echo $LD_LIBRARY_PATH | tr ":" "\n"\n', 'echo =============================\n',
          'echo PATH is\n', 'echo $PATH | tr ":" "\n"\n', 'echo =============================\n',
          'echo MARLIN_DLL is\n', 'echo $MARLIN_DLL | tr ":" "\n"\n', 'echo =============================\n',
          'echo ldd of executable is\n', 'ldd `which Marlin` \n', 'echo =============================\n',
          'echo =============================\n',
          'env | sort >> localEnv.log\n', 'Marlin -c myXML.test testMyArgs\n', 'Marlin myXML.test testMyArgs\n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      remove_mock.assert_called_once_with('Marlin_VT_Run_935.sh')
      assertMockCalls(exists_mock, ['Marlin_VT_Run_935.sh', './lib/lddlib', './lib/marlin_dll',
                                      './lib/lddlib', 'myXML.test', 'testLog.mymarlin', './lib/'], self)
      shell_mock.assert_called_once_with(0, 'sh -c "./Marlin_VT_Run_935.sh"',
                                          callbackFunction=self.marAna.redirectLogOutput,
                                          bufferLimit=20971520)
      chmod_mock.assert_called_once_with('Marlin_VT_Run_935.sh', 0o755)
      appstat_mock.assert_called_once_with('Marlin VT step 935')

  def test_checkRunOverlay(self):
    self.marAna.workflow_commons['OI_2_eventType'] = 'p'
    self.marAna.workflow_commons['OI_1_eventType'] = 'g'

    self.marAna.workflow_commons['OI_2_eventsPerBackgroundFile'] = 2
    self.marAna.workflow_commons['OI_1_eventsPerBackgroundFile'] = 1

    self.marAna.workflow_commons['OI_2_processorName'] = 'P'
    self.marAna.workflow_commons['OI_1_processorName'] = 'G'

    expectedTuple = [('g', 1, 'G'), ('p', 2, 'P')]
    result = self.marAna._checkRunOverlay()
    assertDiracSucceedsWith_equals(result, expectedTuple, self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
 PYTHIA: Second Generator application
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
from DIRAC import S_OK, S_ERROR


class Pythia(LCApplication):
  """Call pythia.

  Usage:

  >>> py = Pythia()
  >>> py.setVersion("tt_500gev_V2")
  >>> py.setEnergy(500) #Can look like a duplication of info, but trust me, it's needed.
  >>> py.setNbEvts(50)
  >>> py.setOutputFile("myfile.stdhep")
  """

  def __init__(self, paramdict=None):
    self.eventType = ''
    super(Pythia, self).__init__(paramdict)
    self.appname = 'pythia'
    self._modulename = 'PythiaAnalysis'
    self._moduledescription = 'Module to run PYTHIA'
    self.datatype = 'gen'

  def willCut(self):
    """You need this if you plan on cutting using :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Applications.StdhepCut`"""
    self.willBeCut = True

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    return m1

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    if not self.version:
      return S_ERROR("Version not specified")

    # Resolve event type, needed for production jobs
    self.eventType = self.version.split("_")[0]

    if not self.numberOfEvents:
      return S_ERROR("Number of events to generate not defined")

    if not self.outputFile:
      return S_ERROR("Output File not defined")

    if not self._jobtype == 'User':
      if not self.willBeCut:
        self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                   "outputDataSE": '@{OutputSE}'})
      self.prodparameters['nbevts'] = self.numberOfEvents
      self.prodparameters['Process'] = self.eventType

    return S_OK()



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Upload generator files to the grid and add metadata::

  dirac-ilc-upload-gen-files -P /some/path/ -E 1000 -M B1b_ws -I 35945 etc.


Options:

  -P, --Path <value>                 Path where the file(s) are (directory or single file)
  -S, --SE <value>                   Storage element(s) to use ex: DESY-SRM
  -M, --MachineParams <value>        Machine Parameters, default: B1b_ws
  -E, --Energy <value>               Energy in GeV, e.g. 1000
  -I, --EvtID <value>                Process ID, like 35945
  -C, --EvtClass <value>             Process class, like 6f_ttbar
  -T, --EvtType <value>              Process type, like 6f_yyyyee
  -L, --Luminosity <value>           Luminosity of the sample
  -N, --NumberOfEvents <value>       Number of events per file
  --BeamParticle1 <value>            Particle of beam 1, e.g. e1
  --BeamParticle2 <value>            Particle of beam 2, e.g. E1
  --PolarisationBeam1 <value>        Polarisation for particle of beam 1: L, R, W
  --PolarisationBeam2 <value>        Polarisation for particle of beam 2: L, R, W
  --XSection <value>                 Cross section in fb
  --XSectionError <value>            Cross section error in fb
  --Software <value>                 Software and version, e.g. whizard-1.95
  -f, --force                        Do not stop for confirmation


:since: Mar 21, 2013
:author: stephane
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"
from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR
import os

MANDATORY_KEYS = ['GenProcessID', 'GenProcessName', 'NumberOfEvents', 'BeamParticle1', 'BeamParticle2',
                  'PolarizationB1', 'PolarizationB2', 'ProgramNameVersion', 'CrossSection', 'CrossSectionError']


class _Params(object):
  def __init__(self):
    self.dir = ''
    self.storageElement = ''
    self.machineParams = 'B1b_ws'
    self.evtclass = ''
    self.evttype = ''
    self.lumi = 0.0
    self.particle1 = 'e'
    self.particle2 = 'p'
    self.pol1 = ''
    self.pol2 = ''
    self.software = 'whizard-1.95'
    self.fmeta = {}
    self.force = False
    self.energy = -1

  def setDir(self, opt):
    self.dir = opt
    return S_OK()

  def setSE(self, opt):
    if opt.count(","):
      return S_ERROR('Cannot use a list of storage elements (yet)')
    else:
      self.storageElement = opt
    return S_OK()

  def setMachineParams(self, opt):
    self.machineParams = opt
    return S_OK()

  def setEnergy(self, opt):
    self.energy = str(opt)
    # check that energy is a simple number
    try:
      _energy = int(opt)
    except ValueError:
      return S_ERROR("Energy should be unit less, only integers")
    return S_OK()

  def setProcessID(self, opt):
    try:
      self.fmeta['GenProcessID'] = int(opt)
    except ValueError:
      return S_ERROR("EvtID MUST be integer")
    return S_OK()

  def setEvtClass(self, opt):
    self.evtclass = opt
    return S_OK()

  def setEvtType(self, opt):
    self.evttype = opt
    self.fmeta['GenProcessName'] = opt
    return S_OK()

  def setLumi(self, opt):
    try:
      self.lumi = float(opt)
    except ValueError:
      return S_ERROR('Luminosity: bad value')
    return S_OK()

  def setNumberOfEvents(self, opt):
    try:
      self.fmeta['NumberOfEvents'] = int(opt)
    except ValueError:
      return S_ERROR('NumberOfEvents: bad value')
    return S_OK()

  def setBeamP1(self, opt):
    self.fmeta['BeamParticle1'] = opt
    if opt == 'e1':
      self.particle1 = 'e'
    elif opt == 'E1':
      self.particle1 = 'p'
    else:
      self.particle1 = opt
    return S_OK()

  def setBeamP2(self, opt):
    self.fmeta['BeamParticle2'] = opt
    if opt == 'e1':
      self.particle2 = 'e'
    elif opt == 'E1':
      self.particle2 = 'p'
    else:
      self.particle2 = opt
    return S_OK()

  def setPol1(self, opt):
    self.fmeta['PolarizationB1'] = opt
    self.pol1 = opt
    return S_OK()

  def setPol2(self, opt):
    self.fmeta['PolarizationB2'] = opt
    self.pol2 = opt
    return S_OK()

  def setSoftware(self, opt):
    self.fmeta['ProgramNameVersion'] = opt
    self.software = opt
    return S_OK()

  def setXSec(self, opt):
    try:
      self.fmeta['CrossSection'] = float(opt)
    except ValueError:
      return S_ERROR("XSection must be float, unit less")
    return S_OK()

  def setXSecE(self, opt):
    try:
      self.fmeta['CrossSectionError'] = float(opt)
    except ValueError:
      return S_ERROR("XSectionError must be float, unit less")
    return S_OK()

  def setForce(self, dummy_opt):
    self.force = True
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch('P:', 'Path=', 'Path where the file(s) are (directory or single file)', self.setDir)
    Script.registerSwitch('S:', "SE=", 'Storage element(s) to use ex: DESY-SRM', self.setSE)
    Script.registerSwitch('M:', "MachineParams=", 'Machine Parameters, default: %s' % self.machineParams,
                          self.setMachineParams)
    Script.registerSwitch("E:", "Energy=", "Energy in GeV, e.g. 1000", self.setEnergy)
    Script.registerSwitch("I:", "EvtID=", "Process ID, like 35945", self.setProcessID)
    Script.registerSwitch("C:", "EvtClass=", "Process class, like 6f_ttbar", self.setEvtClass)
    Script.registerSwitch("T:", "EvtType=", 'Process type, like 6f_yyyyee', self.setEvtType)
    Script.registerSwitch("L:", "Luminosity=", 'Luminosity of the sample', self.setLumi)
    Script.registerSwitch("N:", "NumberOfEvents=", 'Number of events per file', self.setNumberOfEvents)
    Script.registerSwitch('', 'BeamParticle1=', 'Particle of beam 1, e.g. e1', self.setBeamP1)
    Script.registerSwitch('', 'BeamParticle2=', 'Particle of beam 2, e.g. E1', self.setBeamP2)
    Script.registerSwitch('', 'PolarisationBeam1=', 'Polarisation for particle of beam 1: L, R, W', self.setPol1)
    Script.registerSwitch('', 'PolarisationBeam2=', 'Polarisation for particle of beam 2: L, R, W', self.setPol2)
    Script.registerSwitch('', 'XSection=', 'Cross section in fb', self.setXSec)
    Script.registerSwitch('', 'XSectionError=', 'Cross section error in fb', self.setXSecE)
    Script.registerSwitch('', 'Software=', "Software and version, e.g. %s" % self.software, self.setSoftware)
    Script.registerSwitch('f', 'force', "Do not stop for confirmation", self.setForce)
    Script.setUsageMessage('\n%s -P /some/path/ -E 1000 -M B1b_ws -I 35945 etc.\n' % Script.scriptName)


@DIRACScript()
def main():
  """uploads the generator files."""
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()

  from DIRAC import gLogger, exit as dexit

  if not clip.dir:
    gLogger.error('You need to set the path')
    Script.showHelp()
    dexit(1)
  if not clip.storageElement:
    gLogger.error('You need a storage element')
    Script.showHelp()
    dexit(1)

  for key in MANDATORY_KEYS:
    if key not in clip.fmeta:
      gLogger.error("Not all mandatory meta data defined, please check and add key: ", key)
      Script.showHelp()
      dexit(1)

  # resolve the inout files
  flist = []
  if os.path.isdir(clip.dir):
    flistd = os.listdir(clip.dir)
    for filename in flistd:
      if filename.count(".stdhep"):
        flist.append(os.path.join(clip.dir, filename))
  elif os.path.isfile(clip.dir):
    flist.append(clip.dir)
  else:
    gLogger.error("%s is not a file nor a directory" % clip.dir)
    dexit(1)

  gLogger.notice("Will eventually upload %s file(s)" % len(flist))

  from DIRAC.Core.Utilities.PromptUser import promptUser

  from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
  basepath = Operations().getValue('Production/ILC_ILD/BasePath', '')
  if not basepath:
    gLogger.error('Failed to contact CS, please try again')
    dexit(1)

  basepath = "/".join(basepath.split("/")[:-2]) + "/"  # need to get rid of the ild/ part at the end

  finalpath = os.path.join(
      basepath,
      'generated',
      clip.energy
      + "-"
      + clip.machineParams,
      clip.evtclass,
      str(
          clip.fmeta['GenProcessID']))
  gLogger.notice("Will upload the file(s) under %s" % finalpath)
  if not clip.force:
    res = promptUser('Continue?', ['y', 'n'], 'n')
    if not res['OK']:
      gLogger.error(res['Message'])
      dexit(1)
    if not res['Value'].lower() == 'y':
      dexit(0)

  dirmeta = []
  dirmeta.append({'path': os.path.join(basepath, 'generated'), 'meta': {'Datatype': 'gen'}})
  dirmeta.append({'path': os.path.join(basepath, 'generated', clip.energy + "-" + clip.machineParams),
                 'meta': {'Energy': clip.energy, 'MachineParams': clip.machineParams}})
  dirmeta.append({'path': os.path.join(basepath, 'generated', clip.energy + "-"
                 + clip.machineParams, clip.evtclass), 'meta': {'EvtClass': clip.evtclass}})
  dirmeta.append({'path': finalpath, 'meta': {'EvtType': clip.evttype,
                 'Luminosity': clip.lumi, 'ProcessID': clip.fmeta['GenProcessID']}})

  final_fname_base = 'E' + clip.energy + "-" + clip.machineParams + ".P" + \
    clip.fmeta['GenProcessName'] + ".G" + clip.fmeta['ProgramNameVersion'] + "." + clip.particle1 + clip.pol1 + "." + clip.particle2 + clip.pol2 + ".I" + str(clip.fmeta['GenProcessID'])
  gLogger.notice(
      "Final file name(s) will be %s where XX will be replaced by file number, and ext by the input file extension" %
      (final_fname_base + ".XX.ext"))
  if not clip.force:
    res = promptUser('Continue?', ['y', 'n'], 'n')
    if not res['OK']:
      gLogger.error(res['Message'])
      dexit(1)
    if not res['Value'].lower() == 'y':
      dexit(0)

  from DIRAC.DataManagementSystem.Client.DataManager import DataManager
  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
  fc = FileCatalogClient()

  for pathdict in dirmeta:
    res = fc.createDirectory(pathdict['path'])
    if not res['OK']:
      gLogger.error("Could not create this directory in FileCatalog, abort:", pathdict['path'])
      dexit(0)

    res = fc.setMetadata(pathdict['path'], pathdict['meta'])
    if not res['OK']:
      gLogger.error("Failed to set meta data %s to %s\n" % (pathdict['meta'], pathdict['path']), res['Message'])

  datMan = DataManager()
  for filename in flist:
    fnum = filename.split(".")[-2]
    fext = filename.split(".")[-1]
    final_fname = final_fname_base + '.' + fnum + "." + fext
    gLogger.notice("Uploading %s to" % filename, finalpath + "/" + final_fname)
    if not clip.force:
      res = promptUser('Continue?', ['y', 'n'], 'n')
      if not res['OK']:
        gLogger.error(res['Message'])
        break
      if not res['Value'].lower() == 'y':
        break

    res = datMan.putAndRegister(finalpath + "/" + final_fname, filename, clip.storageElement)
    if not res['OK']:
      gLogger.error("Failed to upload %s:" % filename, res['Message'])
      continue
    res = fc.setMetadata(finalpath + "/" + final_fname, clip.fmeta)
    if not res['OK']:
      gLogger.error("Failed setting the metadata to %s:" % filename, res['Message'])

  dexit(0)


if __name__ == '__main__':
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Tests for Interfaces.Utilities.JobHelpers."""

from __future__ import print_function
from __future__ import absolute_import
import unittest

from ILCDIRAC.Interfaces.Utilities import JobHelpers
import six

__RCSID__ = "$Id$"


class TestJobHelpers(unittest.TestCase):
  """tests for the JobHelper utilities."""

  def setUp(self):
    pass

  def tearDown(self):
    pass

  def test_getValue_list_int(self):
    value = ["2", 3]
    ret = JobHelpers.getValue(value, int, int)
    self.assertIsInstance(ret, int)
    self.assertEqual(ret, int(value[0]))

  def test_getValue_int(self):
    value = 2
    ret = JobHelpers.getValue(value, int, int)
    self.assertIsInstance(ret, int)
    self.assertEqual(ret, int(value))

  def test_getValue_int_none(self):
    value = "2"
    ret = JobHelpers.getValue(value, int, None)
    self.assertIsInstance(ret, int)
    self.assertEqual(ret, int(value))

  def test_getValue_string_none(self):
    value = ["someString", "someOther"]
    ret = JobHelpers.getValue(value, str, six.string_types)
    self.assertIsInstance(ret, six.string_types)
    self.assertEqual(ret, value[0])


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestJobHelpers)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()

  # if isinstance( compatmeta['NumberOfEvents'], list ):
  #   self.nbevts = int(compatmeta['NumberOfEvents'][0])
  # else:
  #   #type(compatmeta['NumberOfEvents']) in types.StringTypes:
  #   self.nbevts = int(compatmeta['NumberOfEvents'])



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run the StdHepCutJava utility.

Apply a set of cuts on input stdhep files

:since: Apr 10, 2013

:author: Stephane Poss
"""

from __future__ import absolute_import
import os

from DIRAC import gLogger
from ILCDIRAC.Workflow.Modules.StdHepCut import StdHepCut

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class StdHepCutJava(StdHepCut):
  """Apply cuts on stdhep files, based on L.

  Weuste utility, rewritten in java by C. Grefe.
  """

  def __init__(self):
    super(StdHepCutJava, self).__init__()
    self.applicationName = 'stdhepcutjava'

  def prepareScript(self, mySoftDir):
    """Overloaded from stdhepcuts."""
    self.scriptName = '%s_%s_Run_%s.sh' % (self.applicationName, self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(self.scriptName):
      os.remove(self.scriptName)
    script = open(self.scriptName, 'w')
    script.write('#!/bin/sh \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    if os.path.exists("lib"):
      script.write("declare -x CLASSPATH=./lib:$CLASSPATH\n")
    script.write('echo =========\n')
    script.write('echo java version :\n')
    script.write('java -version\n')
    script.write('env | sort >> localEnv.log\n')
    script.write('echo =============================\n')
    extraopts = ""
    if self.MaxNbEvts:
      extraopts = '-m %s' % self.MaxNbEvts
    comm = 'java -Xmx1536m -Xms256m -jar %s %s -o %s -c %s  %s\n' % (mySoftDir, extraopts,
                                                                     self.OutputFile, self.SteeringFile,
                                                                     self.fileMask)
    LOG.info("Running %s" % comm)
    script.write(comm)
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')
    script.close()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""DB for Overlay System."""
from __future__ import absolute_import
__RCSID__ = "$Id$"

from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.Base.DB import DB
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations


class OverlayDB (DB):
  """DB for OverlaySystem."""

  def __init__(self):
    """"""
    self.ops = Operations()
    self.dbname = 'OverlayDB'
    self.logger = gLogger.getSubLogger('OverlayDB')
    DB.__init__(self, self.dbname, 'Overlay/OverlayDB')
    self._createTables({"OverlayData": {'Fields': {'Site': "VARCHAR(255) UNIQUE NOT NULL",
                                                         'NumberOfJobs': "INTEGER DEFAULT 0"
                                                       },
                                            'PrimaryKey': 'Site',
                                            'Indexes': {'Index': ['Site']}
                                          }
                        }
                      )
    limits = self.ops.getValue("/Overlay/MaxConcurrentRunning", 200)
    self.limits = {}
    self.limits["default"] = limits
    res = self.ops.getSections("/Overlay/Sites/")
    sites = []
    if res['OK']:
      sites = res['Value']
    for tempsite in sites:
      res = self.ops.getValue("/Overlay/Sites/%s/MaxConcurrentRunning" % tempsite, 200)
      self.limits[tempsite] = res
    self.logger.info("Using the following restrictions : %s" % self.limits)

  #####################################################################
  # Private methods

  def __getConnection(self, connection):
    if connection:
      return connection
    res = self._getConnection()
    if res['OK']:
      return res['Value']
    gLogger.warn("Failed to get MySQL connection", res['Message'])
    return connection

  def _checkSite(self, site, connection=False):
    """Check the number of jobs running at a given site."""
    connection = self.__getConnection(connection)

    req = "SELECT NumberOfJobs FROM OverlayData WHERE Site='%s';" % (site)
    res = self._query(req, connection)
    if not res['OK']:
      return S_ERROR("Could not get site")
    if len(res['Value']):
      return res
    else:
      return S_ERROR("Could not find any site %s" % (site))

  def _addSite(self, site, connection=False):
    """Add a new site to the DB."""
    connection = self.__getConnection(connection)
    req = "INSERT INTO OverlayData (Site,NumberOfJobs) VALUES ('%s',1);" % site
    res = self._update(req, connection)
    if not res['OK']:
      return res
    return res

  def _limitForSite(self, site):
    """Get the current limit of jobs for a given site."""
    if site in self.limits:
      return self.limits[site]
    return self.limits['default']

  def _addNewJob(self, site, nbjobs, connection=False):
    """Add a new running job in the DB."""
    connection = self.__getConnection(connection)
    nbjobs += 1
    req = "UPDATE OverlayData SET NumberOfJobs=%s WHERE Site='%s';" % (nbjobs, site)
    self._update(req, connection)
    return S_OK()

# Methods to fix the site
  def getSites(self, connection=False):
    """Return the list of sites known to the service."""
    connection = self.__getConnection(connection)
    req = 'SELECT Site From OverlayData;'
    res = self._query(req, connection)
    if not res['OK']:
      return S_ERROR("Could not get sites")
    sites = []
    for row in res['Value']:
      sites.append(row[0])
    return S_OK(sites)

  def setJobsAtSites(self, sitedict, connection=False):
    """As name suggests: set the number of jobs running at the site."""
    connection = self.__getConnection(connection)
    for site, nbjobs in sitedict.items():
      req = "UPDATE OverlayData SET NumberOfJobs=%i WHERE Site='%s';" % (int(nbjobs), site)
      res = self._update(req, connection)
      if not res['OK']:
        return S_ERROR("Could not set number of jobs at site %s" % site)

    return S_OK()
# Useful methods for the users

  def getJobsAtSite(self, site, connection=False):
    """Get the number of jobs currently run."""
    connection = self.__getConnection(connection)
    nbjobs = 0
    res = self._checkSite(site, connection)
    if not res['OK']:
      return S_OK(nbjobs)
    nbjobs = res['Value'][0][0]
    return S_OK(nbjobs)

# Important methods

  def canRun(self, site, connection=False):
    """Can the job run at that site?"""
    connection = self.__getConnection(connection)
    res = self._checkSite(site, connection)
    nbjobs = 0
    if not res['OK']:
      self._addSite(site, connection)
      nbjobs = 1
    else:
      nbjobs = res['Value'][0][0]
    if nbjobs < self._limitForSite(site):
      res = self._addNewJob(site, nbjobs, connection)
      if not res['OK']:
        return res
      return S_OK(True)
    else:
      return S_OK(False)

  def jobDone(self, site, connection=False):
    """Remove a job from the DB, should not remove a job from the DB if the Site does not exist, but this should never happen."""
    connection = self.__getConnection(connection)
    res = self._checkSite(site, connection)
    if not res['OK']:
      return res
    nbjobs = res['Value'][0][0]
    if nbjobs == 1:
      return S_OK()
    nbjobs -= 1
    req = "UPDATE OverlayData SET NumberOfJobs=%s WHERE Site='%s';" % (nbjobs, site)
    res = self._update(req, connection)
    if not res['OK']:
      return res
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""The CalibrationHandler controls all calibration runs.

The CalibrationHandler collates (calibration results) and distributes (input parameters) information from
the calibration worker nodes and allows the creation of calibration runs. It will (re-)submit jobs and
distribute reconstruction workloads among them.
"""

from __future__ import absolute_import
import os
import re
import math
import shutil
import glob
from datetime import datetime
from datetime import timedelta

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.DISET.RequestHandler import RequestHandler
from DIRAC.Core.Utilities import DErrno
from ILCDIRAC.CalibrationSystem.Service.CalibrationRun import CalibrationRun
from ILCDIRAC.CalibrationSystem.Utilities.fileutils import stringToBinaryFile
from ILCDIRAC.CalibrationSystem.Utilities.functions import loadCalibrationRun
from ILCDIRAC.CalibrationSystem.Utilities.functions import saveCalibrationRun
from ILCDIRAC.CalibrationSystem.Utilities.functions import splitFilesAcrossJobs
from ILCDIRAC.CalibrationSystem.Service.DetectorSettings import CalibrationSettings
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from ILCDIRAC.CalibrationSystem.Utilities.functions import calibration_creation_failed
import six

__RCSID__ = "$Id$"
LOG = gLogger.getSubLogger(__name__)


class WorkerInfo(object):
  """Wrapper class to store information needed by workers to compute their result."""

  def __init__(self, parameterSet, offset):
    """Create a new WorkerInfo object, passed to the worker nodes to enable them to compute their results.

    :param object parameterSet: The histogram to use for the calibration.
    :param int offset: The offset in the event file, used to determine starting point of this computation.
    """
    self.parameterSet = parameterSet
    self.offset = offset

  def getInfo(self):
    """Return worker info."""
    return (self.parameterSet, self.offset)


class CalibrationHandler(RequestHandler):
  """Handles the information exchange between worker nodes and the service."""

  @classmethod
  def initializeHandler(cls, _):
    """Initialize the handler, setting required variables.

    Called once in the beginning of the service.
    """
    cls.activeCalibrations = {}
    cls.idsOfCalibsToBeKilled = []
    cls.calibrationCounter = CalibrationHandler.loadStatus()
    cls.log = LOG
    cls.ops = Operations()
    # FIXME this parameter should be read from CS
    cls.timeToKeepCalibrationResultsInMinutes = cls.ops.getValue('Calibration/timeToKeepCalibrationResultsInMinutes',
                                                                 7 * 24 * 60)  # TODO add this to CS

    # try to find not finished calibrations
    notFinishedCalibIDs = [int(re.findall(r'\d+', x)[0]) for x in glob.glob('calib*')]
    if len(notFinishedCalibIDs) > 0:
      cls.log.info('Recovering calibrations after restart of CalibrationSystem service...')

    for iCalibID in notFinishedCalibIDs:
      tmpCalibRun = loadCalibrationRun(iCalibID)
      if tmpCalibRun is None:
        errMsg = ("Can't recover calibration #%s. No dump file is found in the calibration working directory."
                  " Deteting the directory." % iCalibID)
        cls.log.error(errMsg)
        shutil.rmtree('calib%s' % iCalibID)
      else:
        CalibrationHandler.activeCalibrations[iCalibID] = tmpCalibRun

    if len(notFinishedCalibIDs) > 0:
      cls.log.always('Recovering is finished. Managed to recover following calibrations: %s. Detailed info:' %
                     list(CalibrationHandler.activeCalibrations.keys()))
      for _, iCalib in CalibrationHandler.activeCalibrations.items():
        cls.log.always('Calib #%s: isFinished: %s; stage: %s; phase: %s; step: %s' % (iCalib.calibrationID,
                                                                                      iCalib.calibrationFinished,
                                                                                      iCalib.currentStage,
                                                                                      iCalib.currentPhase,
                                                                                      iCalib.currentStep))

    # TODO ask Andre if we want to stop service or just delete calibratino with ID greate than calibrationCounter?
    if max(notFinishedCalibIDs or [0]) > cls.calibrationCounter:
      errMsg = ('Something went wrong during an attempt to pickup unfinished calibrations during CalibrationHandler'
                ' initialization. calibrationCounter: %s is behind one of the picked up calibration IDs: %s\n Stop the'
                ' service!'
                % (cls.calibrationCounter, notFinishedCalibIDs))
      cls.log.error(errMsg)
      return S_ERROR(errMsg)

    return S_OK()

  def saveStatus(self):
    """Save id of the last calibration to file."""
    fileName = "status"
    with open(fileName, 'w') as f:
      f.write("%s" % self.calibrationCounter)

  @staticmethod
  def loadStatus():
    """Read id of the last calibration from the file."""
    fileName = "status"
    if os.path.exists(fileName):
      with open(fileName, 'r') as f:
        return int(f.readlines()[0])
    else:
      return 0

  def _getUsernameAndGroup(self):
    """Return name of the group and name of the user of the proxy the user is currently having.

    :returns: S_OK with value being dict with 'group' and 'username' entries or S_ERROR
    :rtype: `python:dict`
    """
    credDict = self.getRemoteCredentials()
    if 'DN' not in credDict or 'username' not in credDict:
      return S_ERROR("You must be authenticated!")
    if not credDict['isProxy']:
      return S_ERROR(DErrno.EX509, "chain does not contain a proxy")
    usernameAndGroupDict = {'group': credDict['group'], 'username': credDict['username']}
    return S_OK(usernameAndGroupDict)

  def _checkClientRequest(self, calibId):
    """Check if calibration with input ID exists and if it belongs to the user who made the request.

    :returns: S_OK w/o any message if calibration exists and user credentials are correct
              S_OK with message if calibration with such ID doesn't exist
              S_ERROR if impossible to retrieve user proxy or if user credentials are not correct
    :rtype: `python:dict`
    """
    activeCalibrations = sorted(CalibrationHandler.activeCalibrations.keys())
    if calibId not in activeCalibrations:
      return S_OK('No calibration with ID: %s was found. Active calibrations: %s' % (calibId, activeCalibrations))

    res = self._getUsernameAndGroup()
    if not res['OK']:
      return S_ERROR('Error while retrieving proxy user name or group.')
    usernameAndGroup = res['Value']

    calibration = CalibrationHandler.activeCalibrations[calibId]
    if calibration.proxyUserName == usernameAndGroup['username'] \
       and calibration.proxyUserGroup == usernameAndGroup['group']:
      return S_OK()
    else:
      return S_ERROR('Permission denied. Calibration with ID %s has been created by other user.' % calibId)

  def __checkForRequiredFields(self, requiredFields, providedFields, errMsg=""):
    if set(requiredFields) != set(providedFields):
      errMsg = ("%s Missing fields: [%s]; unused extra fields: [%s]"
                % (errMsg,
                   ','.join(set(requiredFields) - set(providedFields)),
                   ','.join(set(providedFields) - set(requiredFields))))
      self.log.error(errMsg)
      return S_ERROR(errMsg)
    return S_OK()

  # TODO split this function to two: first one just create CalibrationRun instance and returns it (to allow user to
  # setup different settings); second one - submits jobs
  types_createCalibration = [dict, dict, dict]

  def export_createCalibration(self, inputFiles, numberOfEventsPerFile, calibSettingsDict):
    """Create calibration run (series of calibration iterations).

    :param dict inputFiles: Input files for the calibration. Dictionary.
                            Mandatory keys: 'zuds', 'kaon', 'muon', 'gamma'.
    :param dict numberOfEventsPerFile: Number of events per type of input file. Dictionary.
                                       Mandatory keys: 'zuds', 'kaon', 'muon', 'gamma'.
    :param dict calibSettingsDict: Calibration configuration settings reqired to start calibration. Dictionary.
    :returns: S_OK containing ID of the calibration, used to retrieve results etc
    :rtype: dict
    """
    inputFiles = {iKey.lower(): iVal for iKey, iVal in inputFiles.items()}
    numberOfEventsPerFile = {iKey.lower(): iVal for iKey, iVal in numberOfEventsPerFile.items()}

    for iKey, iVal in calibSettingsDict.items():
      keysWithNoneValues = []
      if iVal is None:
        keysWithNoneValues.append(iKey)
      if keysWithNoneValues:
        errMsg = "Following settings have None values: %s. All settings have to be set up." % keysWithNoneValues
        return S_ERROR(errMsg)

    requiredFields = ['gamma', 'kaon', 'muon', 'zuds']
    providedFields = list(inputFiles.keys())
    res = self.__checkForRequiredFields(requiredFields, providedFields,
                                        "Dict of input files doesn't contains all required fields.")
    if not res['OK']:
      return res

    providedFields = list(numberOfEventsPerFile.keys())
    res = self.__checkForRequiredFields(requiredFields, providedFields,
                                                    "Dict of number of events per type of input file doesn't contains"
                                                    " all required fields.")
    if not res['OK']:
      return res

    for iKey, iVal in numberOfEventsPerFile.items():
      if not isinstance(iVal, int) or iVal <= 0:
        errMsg = "Wrong values for numberOfEventsPerFile dict for input file type: %s" % iKey
        self.log.error(errMsg)
        return S_ERROR(errMsg)

    requiredFields = set(CalibrationSettings().settingsDict.keys())
    providedFields = set(calibSettingsDict.keys())
    res = self.__checkForRequiredFields(requiredFields, providedFields,
                                        "Dict of calibration settings doesn't contains all required fields.")
    if not res['OK']:
      return res

    for iKey, iVal in numberOfEventsPerFile.items():
      if iVal * len(inputFiles[iKey]) < calibSettingsDict['numberOfJobs']:
        errMsg = ("number of jobs (%s jobs) is larger than total number of provided events (%s events) for file type:"
                  " %s" % (calibSettingsDict['numberOfJobs'], iVal * len(inputFiles[iKey]), iKey))
        self.log.error(errMsg)
        return S_ERROR(errMsg)

    groupedInputFiles = splitFilesAcrossJobs(inputFiles, numberOfEventsPerFile, calibSettingsDict['numberOfJobs'])

    res = self.__inputVariablesSanityCheck(calibSettingsDict)
    if not res['OK']:
      return res

    if 'platform' in calibSettingsDict:
      errMsg = ('User has provided "platform" settings. It is read from the service configuration now. '
                'Terminate execution')
      self.log.error(errMsg)
      return S_ERROR(errMsg)

    # TODO document that Calibration Service require "Platform" configuration to be set
    calibSettingsDict['platform'] = self.srv_getCSOption('Platform', "x86_64-slc5-gcc43-opt")

    CalibrationHandler.calibrationCounter += 1
    self.saveStatus()
    calibrationID = CalibrationHandler.calibrationCounter

    newRun = CalibrationRun(calibrationID, groupedInputFiles, calibSettingsDict)
    CalibrationHandler.activeCalibrations[calibrationID] = newRun

    res = self._getUsernameAndGroup()
    if not res['OK']:
      return S_ERROR('Error while retrieving proxy user name or group. CalibrationID = %s'
                     % (calibrationID))
    usernameAndGroup = res['Value']
    self.log.info('Retrieve user proxy - userName = %s, userGroup = %s'
                  % (usernameAndGroup['username'], usernameAndGroup['group']))

    newRun.proxyUserName = usernameAndGroup['username']
    newRun.proxyUserGroup = usernameAndGroup['group']

    res = newRun.submitJobs(proxyUserName=newRun.proxyUserName,  # pylint: disable=unexpected-keyword-arg
                            proxyUserGroup=newRun.proxyUserGroup)
    if isinstance(res, dict):
      # if return is a dict then no jobs have been submitted yet
      self.export_killCalibration(calibrationID, 'Error during job submission. Error message: %s' % res['Message'])
      return res

    self.log.info('results from submitJobs: %s' % res)
    if calibration_creation_failed(res):
      ret_val = S_ERROR('Submitting at least one of the jobs failed')
      self.export_killCalibration(calibrationID, 'Submitting at least one of the jobs failed. Calibration is killed.')
      ret_val['calibrations'] = res
      return ret_val
    return S_OK((calibrationID, res))

  types_killCalibrations = [list]

  def export_killCalibrations(self, inList):
    """Kill set of calibrations.

    :returns: S_ERROR if wrong input,
              S_OK with 'Value' parameter being a dict of format: {calibID: <output of killCalibration() call>}
    :rtype: `python:dict`
    """
    outDict = {}
    for iEl in inList:
      if not isinstance(iEl, int):
        errMsg = ('All elements of input list has to be of integer type.'
                  ' You have provided elements of following types: %s' % [type(iEl) for iEl in inList])
        return S_ERROR(errMsg)
    for iEl in inList:
      #  res = self._checkClientRequest(iEl)
      #  if not res['OK'] or res['Value']:
      #    return res
      outDict[iEl] = self.export_killCalibration(iEl, 'killed by user request')
    return S_OK(outDict)

  types_changeDirectoryToCopyTo = [int, str, str]

  def export_changeDirectoryToCopyTo(self, calibId, newPath, newSE):
    """Update "outputPath" settings of the target calibration."""
    if not (isinstance(newPath, str) and isinstance(calibId, int)):
      return S_ERROR('Wrong types of input argumetns. Required types: [int, str]. Provided types: [%s, %s]'
                     % (type(calibId), type(newPath)))

    res = self._checkClientRequest(calibId)
    if not res['OK'] or res['Value']:
      return res

    calibration = CalibrationHandler.activeCalibrations[calibId]
    self.log.info('Calibration #%s: "outputPath" setting is changed from "%s" to "%s"'
                  % (calibId, calibration.settings['outputPath'], newPath))
    self.log.info('Calibration #%s: "outputSE" setting is changed from "%s" to "%s"'
                  % (calibId, calibration.settings['outputSE'], newSE))
    calibration.settings['outputPath'] = newPath
    calibration.settings['outputSE'] = newSE
    calibration.resultsSuccessfullyCopied = False
    return S_OK()

  types_killCalibration = [int, str]

  def export_killCalibration(self, calibIdToKill, errMsg):
    """Kill calibration run.

    Send kill signal to all jobs associated with the calibration; mark calibration as finished; keeps all
    intermediate results on the server in case user want to inspect some of them
    """
    res = self._checkClientRequest(calibIdToKill)
    if not res['OK'] or res['Value']:
      return res

    calibration = CalibrationHandler.activeCalibrations[calibIdToKill]
    if not calibration.calibrationFinished:
      calibration.calibrationFinished = True
      calibration.calibrationEndTime = datetime.now().strftime(calibration.timeStampPattern)
    calibration.resultsSuccessfullyCopied = True  # we don't want files to be copied to user directory
    # if users want logs they can request it with getResults command
    calibration.calibrationRunStatus = errMsg
    saveCalibrationRun(calibration)
    CalibrationHandler.idsOfCalibsToBeKilled += [calibIdToKill]
    return S_OK()

  types_cleanCalibrations = [list]

  def export_cleanCalibrations(self, inList):
    """Remove calibration run from list of active calibrations and delete all associated files."""
    outDict = {}
    for iEl in inList:
      if not isinstance(iEl, int):
        errMsg = ('All elements of input list has to be of integer type.'
                  ' You have provided elements of following types: %s' % [type(iEl) for iEl in inList])
        self.log.error(errMsg)
        return S_ERROR(errMsg)
    for iEl in inList:
      res = self._checkClientRequest(iEl)
      if not res['OK'] or res['Value']:
        return res
      outDict[iEl] = self.export_cleanCalibration(iEl)
    return S_OK(outDict)

  types_cleanCalibration = [int]

  def export_cleanCalibration(self, calibIdToClean):
    """Clean temporary calibration results and remove calibration from the list of active calibrations.

    If calibration is still not finished - do nothing (if user want to stop it he has kill it first).
    """
    res = self._checkClientRequest(calibIdToClean)
    if not res['OK'] or res['Value']:
      return res
    activeCalibrations = sorted(CalibrationHandler.activeCalibrations.keys())
    if calibIdToClean not in activeCalibrations:
      return S_OK('No calibration with ID: %s was found. Active calibrations: %s' % (calibIdToClean,
                                                                                     activeCalibrations))
    calibration = CalibrationHandler.activeCalibrations[calibIdToClean]
    if not calibration.calibrationFinished:
      return S_OK('Cannot clean calibration with ID %s. It is still not finished/killed.' % calibIdToClean)
    else:
      CalibrationHandler.activeCalibrations.pop(calibIdToClean, None)
      shutil.rmtree('calib%s' % calibIdToClean)
      self.log.info('Calibration #%s was cleaned by user.' % calibIdToClean)
      return S_OK('Calibration with ID %s was cleaned.' % calibIdToClean)

  types_getUserCalibrationStatuses = []

  def export_getUserCalibrationStatuses(self):
    """Get status of all active calibration runs.

    Called by user to check status of calibrations.
    """
    res = self._getUsernameAndGroup()
    if not res['OK']:
      return S_ERROR('Error while retrieving proxy user name or group.')
    usernameAndGroup = res['Value']

    statuses = []

    calibList = sorted(CalibrationHandler.activeCalibrations.keys())
    for calibrationID in calibList:
      calibration = CalibrationHandler.activeCalibrations[calibrationID]
      calibBelongsToUser = calibration.proxyUserName == usernameAndGroup['username'] \
                           and calibration.proxyUserGroup == usernameAndGroup['group']
      if not calibBelongsToUser:
        continue
      calibStatus = calibration.getCurrentStatus()
      if 'calibrationEndTime' in calibStatus:
        calibStatus['timeLeftBeforeOutputWillBeDeleted'] = (
            '%s' % (datetime.strptime(calibration.calibrationEndTime, calibration.timeStampPattern)
                    + timedelta(minutes=self.timeToKeepCalibrationResultsInMinutes) - datetime.now()))
      calibStatus['totalNumberOfJobs'] = int(calibration.settings['numberOfJobs'])
      calibStatus['percentageOfFinishedJobs'] = int(
          100.0 * calibration.stepResults[calibration.currentStep].getNumberOfResults()
          / calibration.settings['numberOfJobs'])
      calibStatus['fractionOfFinishedJobsNeededToStartNextStep'] = int(
          100.0 * calibration.settings['fractionOfFinishedJobsNeededToStartNextStep'])
      statuses.append(calibStatus)
    return(S_OK(statuses))

  types_submitResult = [int, int, int, int, int, six.string_types]

  def export_submitResult(self, calibrationID, stageID, phaseID, stepID, workerID, rootFileContent):
    """Report result of the calibration to the service.

    :param int calibrationID: ID of the current calibration run
    :param int phaseID: ID of the stage the calibration is currently in
    :param int phaseID: ID of the phase the calibration is currently in
    :param int stepID: ID of the step in this calibration
    :param int workerID: ID of the reporting worker
    :param rootFileContent: The binary string content of the root file containing the result of the reconstruction run
    :type rootFile: binary data string
    :returns: S_OK in case of success or if the submission was ignored (since it belongs to an older step),
              S_ERROR if the requested calibration can not be found.
    :rtype: dict
    """
    res = self._checkClientRequest(calibrationID)
    if not res['OK'] or res['Value']:
      return res
    calibration = CalibrationHandler.activeCalibrations.get(calibrationID, None)
    if not calibration:
      return S_ERROR('Calibration with ID %d not found.' % calibrationID)
    if stepID is calibration.currentStep:  # Only add result if it belongs to current step. Else ignore (it's ok)
      # FIXME: use mkdir -p like implementation
      dirName = "calib%s/stage%s/phase%s/step%s" % (calibrationID, stageID, phaseID, stepID)
      if not os.path.exists(dirName):
        try:
          os.makedirs(dirName)
        except OSError as e:
          errMsg = 'Cannot create directories. Current working directory: %s. Error message: %s' % (os.getcwd(), e)
          self.log.error(errMsg)
          return S_ERROR(errMsg)

      newFilename = "calib%s/stage%s/phase%s/step%s/pfoanalysis_w%s.root" % (calibrationID, stageID, phaseID, stepID,
                                                                             workerID)
      if stageID == 2:
        newFilename = "calib%s/stage%s/phase%s/step%s/PandoraLikelihoodDataPhotonTraining_w%s.xml" % (calibrationID,
                                                                                                      stageID, phaseID,
                                                                                                      stepID, workerID)
      stringToBinaryFile(rootFileContent, newFilename)
      calibration.addResult(stepID, workerID, newFilename)
    return S_OK()

  types_checkForStepIncrement = []

  def export_checkForStepIncrement(self):
    """Check whether there are any running Calibrations that received enough results to start the next step.

    Should only be called by the agent.
    :returns: S_OK when the check has been ended.
    :rtype: dict
    """
    self.log.info('Executing checkForStepIncrement. activeCalibrations: %s'
                  % list(CalibrationHandler.activeCalibrations.keys()))
    for calibrationID in list(CalibrationHandler.activeCalibrations.keys()):
      # FIXME this still can lead to that some jobs will finish with error status because they didn't finish in time
      calibration = CalibrationHandler.activeCalibrations[calibrationID]
      if calibration.calibrationFinished:
        if not calibration.resultsSuccessfullyCopied:
          res = calibration.copyResults(proxyUserName=calibration.proxyUserName,
                                        proxyUserGroup=calibration.proxyUserGroup) \
              # pylint: disable=unexpected-keyword-arg
          if not res['OK']:
            self.log.error('Cannot copy files of finished calibration', '#%s to user specified destination: %s.'
                           ' Error message: %s' % (calibrationID, calibration.settings['outputPath'], res['Message']))
            return res
          else:
            calibration.resultsSuccessfullyCopied = True
            saveCalibrationRun(calibration)
        if ((datetime.now() - datetime.strptime(calibration.calibrationEndTime, calibration.timeStampPattern)).seconds
              / 60.0 >= self.timeToKeepCalibrationResultsInMinutes):
          if not calibration.resultsSuccessfullyCopied:
            self.log.error('Calibration results have not been copied properly...')
          self.log.info('Removing calibration %s from the active calibration list and clean up local directory')
          CalibrationHandler.activeCalibrations.pop(calibrationID, None)
          shutil.rmtree('calib%s' % calibrationID)
      elif self.finalInterimResultReceived(calibration, calibration.currentStep):
        res = calibration.endCurrentStep()
        if not res['OK']:
          return res
    return S_OK()

  def finalInterimResultReceived(self, calibration, stepID):
    """Check number of finished jobs and start next step if number os larger than a certain threshold.

    Called periodically.
    :param CalibrationRun calibration: The calibration to check
    :param int stepID: The ID of the current step of that calibration
    :returns: True if enough results have been submitted, False otherwise
    :rtype: bool
    """
    # FIXME: Find out of this is susceptible to race condition
    numberOfResults = calibration.stepResults[stepID].getNumberOfResults()
    maxNumberOfJobs = calibration.settings['numberOfJobs']
    startNextStep = numberOfResults >= math.ceil(
        calibration.settings['fractionOfFinishedJobsNeededToStartNextStep'] * maxNumberOfJobs)
    self.log.info('Check status of jobs. CalibID: %d; numberOfResults: %d, totalNumbersOfJobs: %d, startNextStep: %s'
                  % (calibration.calibrationID, numberOfResults, maxNumberOfJobs, startNextStep))
    return startNextStep

  types_getNewParameters = [int, int]

  def export_getNewParameters(self, calibrationID, stepIDOnWorker):
    """Retrieve parameters for the next step of the calibration.

    Called by the worker node.
    :param int calibrationID: ID of the calibration being run on the worker
    :param int stepIDOnWorker: current step ID on the worker node
    :returns: S_ERROR in case of error (e.g. inactive calibration asking for params),
    S_OK with the parameter set and the id of the current step
    :rtype: dict
    """
    cal = CalibrationHandler.activeCalibrations.get(calibrationID, None)
    if not cal:
      self.log.error("CalibrationID is not in active calibrations:",
                     "Active Calibrations:%s , asked for %s" % (self.activeCalibrations,
                                                                calibrationID))
      res = S_ERROR(
          "calibrationID is not in active calibrations: %s\nThis should mean that the calibration has finished"
          % calibrationID)
      return res
    res = self._checkClientRequest(calibrationID)
    if not res['OK'] or res['Value']:
      return res

    res = cal.getNewParameters(stepIDOnWorker)
    return res

  types_getNewPhotonLikelihood = [int]

  def export_getNewPhotonLikelihood(self, calibrationID):
    """Retrieve new photon likelihood file for new step of the calibration.

    Called by the worker node.
    :param int calibrationID: ID of the calibration being run on the worker
    :returns: S_ERROR in case of error (e.g. inactive calibration asking for params),
    S_OK with the parameter set and the id of the current step
    :rtype: dict
    """
    res = self._checkClientRequest(calibrationID)
    if not res['OK'] or res['Value']:
      return res
    cal = CalibrationHandler.activeCalibrations.get(calibrationID, None)
    if not cal:
      self.log.error("CalibrationID is not in active calibrations:",
                     "Active Calibrations:%s , asked for %s" % (self.activeCalibrations,
                                                                calibrationID))
      result = S_ERROR(
          "calibrationID is not in active calibrations: %s\nThis should mean that the calibration has finished"
          % calibrationID)
    else:
      result = cal.getNewPhotonLikelihood()
    return result

  types_getInputDataDict = [int, int]

  def export_getInputDataDict(self, calibrationID, workerID):
    """Retrieve dict with input files and number of events to run and to skip.

    Called by the worker node.
    :param int calibrationID: ID of the calibration being run on the worker
    :returns: S_ERROR in case of error (e.g. inactive calibration asking for params),
    S_OK with the parameter set and the id of the current step
    :rtype: dict
    """
    res = self._checkClientRequest(calibrationID)
    if not res['OK'] or res['Value']:
      return res
    cal = CalibrationHandler.activeCalibrations.get(calibrationID, None)
    if not cal:
      self.log.error("CalibrationID is not in active calibrations:",
                     "Active Calibrations:%s , asked for %s" % (self.activeCalibrations,
                                                                calibrationID))
      result = S_ERROR(
          "calibrationID is not in active calibrations: %s\nThis should mean that the calibration has finished"
          % calibrationID)
    else:
      if workerID >= cal.settings['numberOfJobs']:
        errMsg = ('Value of workerID is larger than number of job in this calibration: '
                  'calibID: %s, nJobs: %s, workerID: %s' % (calibrationID, cal.settings['numberOfJobs'], workerID))
        self.log.error(errMsg)
        result = S_ERROR(errMsg)
      else:
        result = S_OK(cal.inputFiles[workerID])

    return result

  types_resubmitJobs = [list]

  def export_resubmitJobs(self, failedJobs):
    """Resubmit failed jobs of running calibrations.

    Called by the Agent.
    :param failedJobs: List of pairs of the form (calibrationID, workerID)
    :type failedJobs: `python:list`
    :returns: S_OK if successful, else a S_ERROR with a pair (errorstring, list_of_failed_id_pairs)
    :rtype: dict
    """
    res = self.export_getRunningCalibrations()
    if not res['OK']:
      return res
    runnigCalibs = res['Value']
    for iCalib in runnigCalibs:
      jobsToResubmit = []
      for calibrationID, workerID in failedJobs:
        if calibrationID == iCalib:
          jobsToResubmit.append(workerID)
      if jobsToResubmit:
        calibRun = CalibrationHandler.activeCalibrations[iCalib]
        calibRun.nFailedJobs += len(jobsToResubmit)
        if calibRun.nFailedJobs < calibRun.settings['numberOfJobs']:
          calibRun.submitJobs(jobsToResubmit, proxyUserName=calibRun.proxyUserName,
                              proxyUserGroup=calibRun.proxyUserGroup)  # pylint: disable=unexpected-keyword-arg
        else:
          errMsg = ('Number of failed jobs are larger than total number of jobs. Something wrong with this calibration'
                    ' run. Kill it!')
          # TODO FIXME test it! initial test failed...
          self.log.error(errMsg)
          res = self.export_killCalibration(iCalib, errMsg)
          if not res['OK']:
            return res
    return S_OK()

  types_getNumberOfJobsPerCalibration = []

  def export_getNumberOfJobsPerCalibration(self):
    """Return a dictionary that maps active calibration IDs to the number of initial jobs they submitted.

    Used by the agent to determine when to resubmit jobs.
    :returns: S_OK containing the dictionary with mapping calibrationID -> numberOfJobs
    :rtype: dict
    """
    result = {}
    for calibrationID in CalibrationHandler.activeCalibrations:
      result[calibrationID] = CalibrationHandler.activeCalibrations[calibrationID].settings['numberOfJobs']
    self.log.debug("Number of jobs per calibration: %s" % result)
    return S_OK(result)

  types_getRunningCalibrations = []

  def export_getRunningCalibrations(self):
    """Return a list of unfinished calibrations.

    :returns: S_OK containing the the list of calibrationIDs
    :rtype: list
    """
    result = []
    for calibrationID in CalibrationHandler.activeCalibrations:
      if not CalibrationHandler.activeCalibrations[calibrationID].calibrationFinished:
        result.append(calibrationID)
    self.log.debug("List of running calibrations: %s" % result)
    return S_OK(result)

  types_getActiveCalibrations = []

  def export_getActiveCalibrations(self):
    """Return a list of all active calibrations.

    :returns: S_OK containing the the list of calibrationIDs
    :rtype: list
    """
    self.log.debug("List of active calibrations: %s" % CalibrationHandler.activeCalibrations)
    activeCalibrations = list(CalibrationHandler.activeCalibrations.keys())
    return S_OK(activeCalibrations)

  types_getCalibrationsToBeKilled = []

  def export_getCalibrationsToBeKilled(self):
    """Return list of calibrations to be killed."""
    listToReturn = CalibrationHandler.idsOfCalibsToBeKilled
    CalibrationHandler.idsOfCalibsToBeKilled = []
    self.log.debug("List of calibrations to be killed: %s" % listToReturn)
    return S_OK(listToReturn)

# TODO: Add stopping criterion to calibration loop. This should be checked when new parameter sets are calculated
# In that case, the calibration should be removed from activeCalibrations and the result stored.
# Should we then kill all jobs of that calibration?

  def __inputVariablesSanityCheck(self, inputSettings):

    settingsDictCopy = dict(inputSettings)

    def checkType(inType, containerType=None):
      errMsg = 'Invalid type of input settings for argument: %s'
      vals = {key: settingsDictCopy[key] for key in keys}
      for key, val in vals.items():
        if containerType is None:
          if not isinstance(val, inType):
            return errMsg % key
        else:
          for containerElement in val:
            if not isinstance(containerElement, inType):
              return errMsg % key
      return None

    keys = ['disableSoftwareCompensation', 'startCalibrationFinished']
    msg = checkType(bool)
    if msg:
      return S_ERROR(msg)

    keys = ['startStage', 'startPhase', 'stopStage', 'stopPhase', 'nHcalLayers']
    msg = checkType(int)
    if msg:
      return S_ERROR(msg)

    keys = ['digitisationAccuracy', 'pandoraPFAAccuracy', 'fractionOfFinishedJobsNeededToStartNextStep']
    msg = checkType(float)
    if msg:
      return S_ERROR(msg)

    keys = ['marlinVersion', 'DDPandoraPFANewProcessorName', 'DDCaloDigiName', 'nameOfTheConfigPackage',
            'detectorModel', 'steeringFile', 'DDPandoraPFANewProcessorName', 'DDCaloDigiName']
    msg = checkType(str)
    if msg:
      return S_ERROR(msg)

    keys = ['ecalBarrelCosThetaRange', 'ecalEndcapCosThetaRange', 'hcalBarrelCosThetaRange', 'hcalEndcapCosThetaRange']
    msg = checkType(float, list)
    if msg:
      return S_ERROR(msg)

    def checkVal(validRange, isList=False):
      errMsg = 'Invalid value of input settings for argument: %s;'
      errMsg += ' Valid value range: %s;' % validRange
      errMsg += ' Provided value: %s'
      vals = {key: settingsDictCopy[key] for key in keys}
      for key, val in vals.items():
        if not isList:
          if val < validRange[0] or val > validRange[1]:
            return errMsg % (key, val)
        else:
          for containerElement in val:
            if containerElement < validRange[0] or containerElement > validRange[1]:
              return errMsg % (key, containerElement)
      return None

    keys = ['startStage']
    msg = checkVal([1, 3])
    if msg:
      return S_ERROR(msg)

    keys = ['startPhase']
    msg = checkVal([0, 5])
    if msg:
      return S_ERROR(msg)

    keys = ['digitisationAccuracy', 'pandoraPFAAccuracy', 'fractionOfFinishedJobsNeededToStartNextStep']
    msg = checkVal([0.0, 1.0])
    if msg:
      return S_ERROR(msg)

    keys = ['ecalBarrelCosThetaRange', 'ecalEndcapCosThetaRange', 'hcalBarrelCosThetaRange', 'hcalEndcapCosThetaRange']
    msg = checkVal([0.0, 1.0], isList=True)
    if msg:
      return S_ERROR(msg)

    self.log.debug('All input variables are valid.')
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Print production properties and more information for given production.

Example::

  dirac-ilc-get-info -p 5678

Options:
  -p, --ProductionID prodID      ProductionID
  -f, --File lfn                 LFN from the Production
"""
from __future__ import absolute_import
import pprint

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, exit as dexit

__RCSID__ = "$Id$"


class _Params(object):
  """CLI parameters."""

  def __init__(self):
    self.filename = ""
    self.prodid = 0

  def setFilename(self, opt):
    self.filename = opt
    return S_OK()

  def setProdID(self, opt):
    try:
      self.prodid = int(opt)
    except ValueError:
      return S_ERROR('Prod ID must be integer')
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch('p:', "ProductionID=", "Production ID", self.setProdID)
    Script.registerSwitch('f:', "File=", "File name", self.setFilename)
    Script.setUsageMessage("%s -p 12345" % Script.scriptName)


def _createTransfoInfo(trans):
  """creates information for Transformation."""
  info = []
  parametersToShow = ['Type', 'LongDescription', 'TransformationGroup', 'Status',
                      'Plugin', 'TransformationName', 'AuthorDN', 'InputDataQuery']
  maxLength = len(max(parametersToShow + list(trans['AddParams'].keys()), key=len))
  for key in parametersToShow:
    if key in trans:
      info.append("    %s: %s" % (key.ljust(maxLength), trans[key]))
  for key, val in trans.get('AddParams', {}).items():
    if key in ['DetailedInfo', 'JobType', 'FCInputQuery', 'detectorType', 'NbInputFiles', 'nbevts', 'Energy'] \
       or any(key.endswith(ext) for ext in ('slcio', 'stdhep')):
      continue
    if 'whizardparams' in key:
      pp = pprint.PrettyPrinter(indent=4)
      whizp = pp.pformat(eval(val))
      info.append(" - Uses the following whizard parameters:")
      info.append("      %s" % whizp)
    elif 'SinFile' in key:
      # strip empty lines from the file
      sinFile = [line for line in val.splitlines() if line]
      info.append("    %s: %s" % (key.ljust(maxLength),
                                  ("\n" + " " * (maxLength + 6)).join(sinFile)))
    else:
      info.append("    %s: %s" % (key.ljust(maxLength), val))

  info.append("")
  return info


def _createFileInfo(fmeta):
  """creates information for file."""
  from DIRAC.Core.Utilities import DEncode
  fmeta.pop('ProdID', None)

  info = []

  parametersToShow = ['Machine', 'Energy', 'MachineParams', 'EvtClass', 'ProcessID', 'GenProcessID',
                      'EvtType', 'Polarisation', 'BeamParticle1', 'BeamParticle2',
                      'PolarisationB1', 'PolarisationB2', 'Datatype',
                      'SWPackages', 'SoftwareTag', 'ILDConfig', 'DetectorModel', 'NumberOfEvents',
                      'CrossSection'
                     ]
  datatypes = dict(GEN="Generator Sample", SIM="Simulated Sample",
                   REC="Reconstructed Sample", DST="Reconstructed Sample")
  maxLength = len(max(parametersToShow, key=len))

  for parameter in parametersToShow:
    if parameter not in fmeta:
      continue

    value = fmeta[parameter]
    if parameter == 'Datatype':
      value = datatypes.get(value.upper(), ("Unknown Datatype"))

    if parameter == 'Energy':
      value += " GeV"

    if parameter == 'CrossSection':
      value = str(value) + " fb"
      if 'CrossSectionError' in fmeta:
        value += "+/-" + str(fmeta["CrossSectionError"]) + "fb"
        del fmeta['CrossSectionError']

    info.append("    %s: %s" % (parameter.ljust(maxLength), value))
    del fmeta[parameter]

  if "AdditionalInfo" in fmeta:
    try:
      dinfo = DEncode.decode(fmeta["AdditionalInfo"])
    except Exception:  # cannot do anything else because decode raises base Exception #pylint: disable=W0703
      dinfo = eval(fmeta["AdditionalInfo"])
    info.append(" - There is some additional info:")
    if isinstance(dinfo, tuple):
      dinfo = dinfo[0]
    if isinstance(dinfo, dict):
      dictinfo = dinfo
      if 'xsection' in dictinfo and 'sum' in dictinfo['xsection'] and 'xsection' in dictinfo['xsection']['sum']:
        xsec = str(dictinfo['xsection']['sum']['xsection'])
        if 'err_xsection' in dictinfo['xsection']['sum']:
          xsec += ' +/- %s' % dictinfo['xsection']['sum']['err_xsection']
        xsec += "fb"
        info.append('    Cross section %s' % xsec)

    else:
      info.append('    %s' % dinfo)

    del fmeta["AdditionalInfo"]

  if 'Luminosity' in fmeta:
    info.append(' - Sample corresponds to a luminosity of %sfb' % fmeta["Luminosity"])
    del fmeta['Luminosity']

  if 'Ancestors' in fmeta:
    if len(fmeta["Ancestors"]):
      info.append(" - Was produced from:")
      for anc in fmeta["Ancestors"]:
        info.append('    %s' % anc)
    del fmeta["Ancestors"]

  if 'Descendants' in fmeta:
    if len(fmeta["Descendants"]):
      info.append(" - Gave the following files:")
      for des in fmeta["Descendants"]:
        info.append('    %s' % des)
    del fmeta["Descendants"]

  if 'DetectorType' in fmeta:
    # We don't need this here
    del fmeta['DetectorType']

  if fmeta:
    info.append('Remaining file metadata: %s' % str(fmeta))

  return info


@DIRACScript()
def main():
  """gets info about transformation."""
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()

  if not clip.prodid and not clip.filename:
    Script.showHelp()
    dexit(1)

  from DIRAC import gLogger
  import os

  from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
  tc = TransformationClient()

  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
  fc = FileCatalogClient()
  fmeta = {}
  trans = None
  info = []

  if clip.prodid:
    res = tc.getTransformation(clip.prodid)
    if not res['OK']:
      gLogger.error(res['Message'])
      dexit(1)
    trans = res['Value']
    res = tc.getTransformationMetaDataQuery(clip.prodid, 'Input')
    if res['OK']:
      trans['InputDataQuery'] = res['Value']
    res = tc.getAdditionalParameters(clip.prodid)
    if res['OK']:
      trans['AddParams'] = res['Value']
    # do something with transf
    res1 = fc.findDirectoriesByMetadata({'ProdID': clip.prodid})
    if res1['OK'] and len(list(res1['Value'].values())):
      gLogger.verbose("Found %i directory matching the metadata" % len(list(res1['Value'].values())))
      for dirs in res1['Value'].values():
        res = fc.getDirectoryUserMetadata(dirs)
        if res['OK']:
          fmeta.update(res['Value'])
        else:
          gLogger.error("Failed to get metadata for %s, SKIPPING" % dirs)
          continue
        res = fc.listDirectory(dirs)
        if not res['OK']:
          continue
        content = res['Value']['Successful'][dirs]
        if content["Files"]:
          for f_ex in content["Files"].keys():
            res = fc.getFileUserMetadata(f_ex)
            if res['OK']:
              fmeta.update(res['Value'])
              break

    # here we have trans and fmeta
    info.append("")
    info.append("Production %s has the following parameters:" % trans['TransformationID'])
    info.extend(_createTransfoInfo(trans))

    if fmeta:
      info.append('The files created by this production have the following metadata:')
      info.extend(_createFileInfo(fmeta))
      info.append("It's possible that some meta data was not brought back,")
      info.append("in particular file level metadata, so check some individual files")

  if clip.filename:
    pid = ""
    if clip.filename.count("/"):
      fpath = os.path.dirname(clip.filename)
      res = fc.getDirectoryUserMetadata(fpath)
      if not res['OK']:
        gLogger.error(res['Message'])
        dexit(0)
      fmeta.update(res['Value'])
      res = fc.getFileUserMetadata(clip.filename)
      if not res['OK']:
        gLogger.error(res['Message'])
        dexit(1)
      fmeta.update(res['Value'])
      if 'ProdID' in fmeta:
        pid = str(fmeta['ProdID'])
      res = fc.getFileAncestors([clip.filename], 1)
      if res["OK"]:
        for dummy_lfn, ancestorsDict in res['Value']['Successful'].items():
          if ancestorsDict:
            fmeta["Ancestors"] = list(ancestorsDict.keys())
      res = fc.getFileDescendents([clip.filename], 1)
      if res["OK"]:
        for dummy_lfn, descendDict in res['Value']['Successful'].items():
          if descendDict:
            fmeta['Descendants'] = list(descendDict.keys())
    else:
      ext = clip.filename.split(".")[-1]
      fitems = []
      for i in clip.filename.split('.')[:-1]:
        fitems.extend(i.split('_'))
      pid = ''
      if ext == 'stdhep':
        pid = fitems[fitems.index('gen') + 1]
      if ext == 'slcio':
        if 'rec' in fitems:
          pid = fitems[fitems.index('rec') + 1]
        elif 'dst' in fitems:
          pid = fitems[fitems.index('dst') + 1]
        elif 'sim' in fitems:
          pid = fitems[fitems.index('sim') + 1]
        else:
          gLogger.error("This file does not follow the ILCDIRAC production conventions!")
          gLogger.error("Please specify a prod ID directly or check the file.")
          dexit(0)
      if not pid:
        gLogger.error("This file does not follow the ILCDIRAC production conventions!")
        gLogger.error("Please specify a prod ID directly or check the file.")
        dexit(0)
      # as task follows the prod id, to get it we need
      tid = fitems[fitems.index(pid) + 1]
      last_folder = str(int(tid) // 1000).zfill(3)
      res = fc.findDirectoriesByMetadata({'ProdID': int(pid)})
      if not res['OK']:
        gLogger.error(res['Message'])
        dexit(1)
      dir_ex = list(res['Value'].values())[0]
      fpath = ""
      if int(dir_ex.split("/")[-1]) == int(pid):
        fpath = dir_ex + last_folder + "/"
      elif int(dir_ex.split("/")[-2]) == int(pid):
        fpath = "/".join(dir_ex.split('/')[:-2]) + "/" + pid.zfill(8) + "/" + last_folder + "/"
      else:
        gLogger.error('Path does not follow conventions, will not get file family')

      if fpath:
        fpath += clip.filename
        res = fc.getFileAncestors([fpath], 1)
        if res["OK"]:
          for dummy_lfn, ancestorsDict in res['Value']['Successful'].items():
            fmeta["Ancestors"] = list(ancestorsDict.keys())
        res = fc.getFileDescendents([fpath], 1)
        if res["OK"]:
          for dummy_lfn, descendDict in res['Value']['Successful'].items():
            fmeta['Descendants'] = list(descendDict.keys())

      res = fc.getDirectoryUserMetadata(dir_ex)
      if not res['OK']:
        gLogger.error(res['Message'])
      else:
        fmeta.update(res['Value'])
    res = tc.getTransformation(pid)
    if not res['OK']:
      gLogger.error(res['Message'])
      gLogger.error('Will proceed anyway')
    else:
      trans = res['Value']
      res = tc.getTransformationMetaDataQuery(pid, 'Input')
      if res['OK']:
        trans['InputDataQuery'] = res['Value']
      res = tc.getAdditionalParameters(pid)
      if res['OK']:
        trans['AddParams'] = res['Value']
    info.append("")
    info.append("Input file has the following properties:")
    info.extend(_createFileInfo(fmeta))
    info.append("")
    info.append('It was created with the production %s:' % pid)
    if trans:
      info.extend(_createTransfoInfo(trans))

  gLogger.notice("\n".join(info))

  dexit(0)


if __name__ == "__main__":
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Print all applications and their versions registered in the configuration service.

Options:

   -S, --software software       show versions for this application only
   -A, --apps                    show only available Applications, not their versions

:since: Dec 17, 2010
:author: sposs
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"

import os

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK


class _Params(object):
  """Parameter Object."""

  def __init__(self):
    self.software = ''
    self.appsOnly = False

  def setSoftware(self, opt):
    self.software = opt
    return S_OK()

  def setAppsOnly(self, _):
    self.appsOnly = True
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch("S:", "software=", "show versions for this software", self.setSoftware)
    Script.registerSwitch("A", "apps", "show only available Applications, not their versions", self.setAppsOnly)
    Script.setUsageMessage("""%s """ % ("dirac-ilc-show-software",))


@DIRACScript()
def main():
  """Show available software."""
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()
  from DIRAC import gConfig, gLogger

  base = '/Operations/Defaults/AvailableTarBalls'
  platforms = gConfig.getSections(base)

  for platform in platforms['Value']:
    gLogger.notice("For platform %s, here are the available software" % platform)
    apps = gConfig.getSections(base + "/" + platform)
    for app in apps['Value']:
      if clip.software and app.lower() != clip.software.lower():
        continue
      gLogger.notice("   - %s" % app)
      versions = gConfig.getSections(base + "/" + platform + "/" + app)
      if clip.appsOnly:
        continue
      for vers in versions['Value']:
        gLogger.notice("     * %s" % vers)
        depsb = gConfig.getSections(base + "/" + platform + "/" + app + "/" + vers)
        if 'Dependencies' in depsb['Value']:
          gLogger.notice("       Depends on")
          deps = gConfig.getSections(os.path.join(base, platform, app, vers, "Dependencies"))
          for dep in deps['Value']:
            depversions = gConfig.getOption(base + "/" + platform + "/" + app + "/"
                                            + vers + "/Dependencies/" + dep + "/version")
            gLogger.notice("         %s %s" % (dep, depversions['Value']))

      if not len(versions['Value']):
        gLogger.notice("      No version available")


if __name__ == "__main__":
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run Pythia, but only specific versions (the CDR ttbar ones)

:since:  Jun 3, 2011

:author: Stephane Poss
"""

from __future__ import absolute_import
from DIRAC.Core.Utilities.Subprocess import shellCall
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase, generateRandomString
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import getNewLDLibs
from ILCDIRAC.Core.Utilities.ResolveDependencies import resolveDeps
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename
from DIRAC import gLogger, S_OK, S_ERROR

import os

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class PythiaAnalysis(ModuleBase):
  """Run pythia.

  Used for CDR vol2, but not for vol3: easier to produce the files locally.
  """

  def __init__(self):
    super(PythiaAnalysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.debug = True

  def applicationSpecificInputs(self):

    if 'IS_PROD' in self.workflow_commons:
      if self.workflow_commons["IS_PROD"]:
        # self.OutputFile = getProdFilename(self.OutputFile,int(self.workflow_commons["PRODUCTION_ID"]),
        #                                  int(self.workflow_commons["JOB_ID"]))
        if 'ProductionOutputData' in self.workflow_commons:
          outputlist = self.workflow_commons['ProductionOutputData'].split(";")
          baseoutputfile = self.OutputFile.split(".stdhep")[0]
          for obj in outputlist:
            if obj.count(baseoutputfile):
              self.OutputFile = os.path.basename(obj)
        else:
          self.OutputFile = getProdFilename(self.OutputFile,
                                            int(self.workflow_commons["PRODUCTION_ID"]),
                                            int(self.workflow_commons["JOB_ID"]))

    return S_OK()

  def execute(self):
    """Run the module."""
    self.result = self.resolveInputVariables()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationName:
      self.result = S_ERROR("Pythia version name not given")
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve the input parameters:", self.result["Message"])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('%s should not proceed as previous step did not end properly' % self.applicationName)

    res = getSoftwareFolder(self.platform, self.applicationName, self.applicationVersion)
    if not res['OK']:
      LOG.error('Failed finding the software area')
      self.setApplicationStatus('Could find neither local area nor shared area install')
      return res
    myappDir = res['Value']

    deptar = resolveDeps(self.platform, self.applicationName, self.applicationVersion)[0]
    res = getSoftwareFolder(self.platform, deptar["app"], deptar["version"])
    if not res['OK']:
      LOG.error("Failed finding the dependency location")
      return res
    path = res['Value']
    if not os.path.exists("%s.ep" % path):
      LOG.error('Could not find the lumi files!')
      return S_ERROR("Lumi files not found")

    originpath = "%s.ep" % path
    randomName = '/tmp/LumiFile-' + generateRandomString(8)
    try:
      os.mkdir(randomName)
    except EnvironmentError as x:
      LOG.error("Failed setting up the temp directory")
      return S_ERROR("Could not create temp dir: %s" % str(x))

    try:
      os.symlink(originpath, "%s/%s" % (randomName, os.path.basename(originpath)))
    except EnvironmentError as why:
      LOG.error('Failed setting up the sym link to lumi files')
      return S_ERROR("Cannot sym link lumi file: %s %s" % str(why))
    # try :
    #  shutil.copy(originpath,"/tmp/")
    # except:
    #  return S_ERROR("Could not copy to /tmp")
    #self.lumifile = path+"/%s.ep"%depdir
    lumifile = "%s/%s" % (randomName, os.path.basename(originpath).replace(".ep", ""))
    # Need to fetch the new LD_LIBRARY_PATH
    new_ld_lib_path = getNewLDLibs(self.platform, self.applicationName, self.applicationVersion)
    new_ld_lib_path = myappDir + "/lib:" + new_ld_lib_path

    scriptName = '%s_%s_Run_%s.sh' % (self.applicationName, self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/sh \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    if new_ld_lib_path:
      script.write('declare -x LD_LIBRARY_PATH=%s\n' % new_ld_lib_path)
    script.write("declare -x NBEVTS=%s\n" % self.NumberOfEvents)
    script.write("declare -x LumiFile=%s\n" % lumifile)
    script.write("declare -x OUTPUTFILE=%s\n" % self.OutputFile)
    script.write('echo ======================================\n')
    script.write('env | sort >> localEnv.log\n')
    comm = "%s/%s_%s.exe\n" % (myappDir, self.applicationName, self.applicationVersion)
    LOG.info("Will run %s" % comm)
    script.write(comm)
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')
    script.close()
    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % scriptName
    self.setApplicationStatus('%s %s step %s' % (self.applicationName, self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    if not self.result['OK']:
      LOG.error('Something wrong during running:', self.result['Message'])
      self.setApplicationStatus('Error during running %s' % self.applicationName)
      return S_ERROR('Failed to run %s' % self.applicationName)

    #self.result = {'OK':True,'Value':(0,'Disabled Execution','')}
    resultTuple = self.result['Value']
    status = resultTuple[0]

    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed terribly, you are doomed!' % (self.applicationName))
      if not self.ignoreapperrors:
        return S_ERROR('%s did not produce the expected log' % (self.applicationName))

    if os.path.exists("pythiaGen.lpt"):
      base = "pythiaGen.lpt".split(".lpt")[0]
      try:
        os.rename("pythiaGen.lpt", base + self.STEP_NUMBER + ".lpt")
      except EnvironmentError:
        LOG.error("Could not rename, deleting")
        os.unlink("pythiaGen.lpt")

    with open(self.applicationLog) as logf:
      success = False
      for line in logf:
        if line.count("Evts Generated= "):
          success = True
      if not success:
        status = 1

    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Root Macro Application: use a macro in the Root application framework."""
from __future__ import absolute_import
__RCSID__ = "$Id"

from ILCDIRAC.Interfaces.API.NewInterface.Applications import _Root
from DIRAC import S_OK
import os
import types


class RootMacro(_Root):
  """Run a root macro in the root application environment.

  Example:

  >>> rootmac = RootMacro()
  >>> rootmac.setMacro("mymacro.C")
  >>> rootmac.setArguments("some command line arguments")

  The :func:`setExtraCLIArguments` is not available here, use the :func:`setArguments`
  """

  def __init__(self, paramdict=None):
    super(RootMacro, self).__init__(paramdict)
    self._modulename = "RootMacroAnalysis"
    self.appname = 'root'
    self._moduledescription = 'Root macro execution'

  def setMacro(self, macro):
    """Define macro to use.

    :param str macro: Macro to run on. Must be a local C file.
    """
    self._checkArgs({'macro': (str,)})

    self.script = macro
    if os.path.exists(macro) or macro.lower().count("lfn:"):
      self.inputSB.append(macro)
    return S_OK()
